<?php

namespace Teknisa\DF\Modules\Entrada\Services;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Factories\Dfe51000ImportacaoConsultaNotaFiscalFactory;
use Teknisa\DF\Services\NFe;
use Teknisa\DF\Services\NFe\ReturnNFe;
use Teknisa\DF\Services\NFe\Danfe;

class ImportacaoConsultaNotaFiscalService {

    const FILESERVER_LINK   = 'http://midia.teknisa.com/files/';
    const PROCEDURE_GERA_NF = 'GERA_NOTAFISCAL_CORPO_NFE';
    const IMPORTACAO_XML    = 'Importação de Arquivo XML';
    const GERA_XML          = 'Re-criação de arquivo XML';
    const MSG_ERROS         = array(
            "SEM_DIR" => "Diretório parametrizado não existe",
            "DIR_INV" => "Não há diretório parametrizado. Favor indicar o diretório onde estão os arquivos a serem
                            importados na tela de Parâmetros de Integração.",
            "IMP_ASO" => "Impostos da nota fiscal eletrônica devem ser associados.",
            "IMP_FAT" => "Impostos da Fatura Eletrônica não associado",
            "ENC_ASO" => "Todos os encargos da nota fiscal eletrônica devem ser associados.",
            "NAO_NOT" => "Não há notas no diretório."
        );
    
    const ERROS_IMPORTACAO = array(
            "ARQ_INV"          => "Alguns Arquivos foram importados com sucesso, mas existem arquivos inválidos 
                            que não foram importados.",
            "ERR_DESTINATARIO" => "Nota fiscal não pertence ao destinatário",
            "ERR_INS"          => "Nota já importada",
            "ERR_FORN"         => "Fornecedor %s não Cadastrado.", 
            "ERR_FORN_BLOQ"    => "Fornecedor %s não utiliza XML. Parâmetros de Integração.",
            "ERR_SERIE"        => "Série %s não cadastrada.",
            "ERR_MODDOCFISC"   => "Modelo de Documento %s não cadastrado.",
            "ERROS_MANIF"      => array( "ERR_CAN" => "Nota fiscal cancelada na receita federal.",
                                    "ERR_DEN" => "Nota fiscal denegada na receita federal.",
                                    "ERR_DES" => "Operação Bloqueada! Nota fiscal com manifestação Desconhecimento da Operação. Para".
                                                 " inclussão desta nota é preciso alterar a manifestação do úsuario.",
                                    "ERR_NAP" => "Nota fiscal não aprovada pela receita federal.",
                                    "ERR_NRE" => "Operação Bloqueada! Nota fiscal com manifestação Operação não realizada. Para".
                                                 " inclussão desta nota é preciso alterar a manifestação do úsuario.",
                                    "XML_DIF" => "XML com dados distintos da SEFAZ.",
                                    "NAO_POS" => "Não foi possivel consultar a situação da nota fiscal.")
        
            
    );

    protected $baseDir;
    protected $connection;
    protected $dfe51000ImportacaoConsultaNotaFiscalFactory;
    protected $diretorioImportacao;
    protected $entityManager;
    protected $environment;
    protected $manifestacaoService;
    protected $fileServerUri;
    protected $smtpService;
    protected $smtpParams;
    protected $SAAS;
    
    private $erros = array();

    public function __construct(EntityManager $entityManager, $environment, $dfe51000ImportacaoConsultaNotaFiscalFactory, $manifestacaoService, $fileServerBaseUrl, $smtpService, $smtpParams) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory = $dfe51000ImportacaoConsultaNotaFiscalFactory;
        $this->baseDir    = str_replace("erp/modules/df/backend/service", "anexos/", getcwd());
        $this->manifestacaoService = $manifestacaoService;
        $this->connection    = $entityManager->getConnection();

        if($this->environment->getNrOrgTrab() == 1) {
            $this->SAAS = false;
        }
        else {
            $this->SAAS = true;
        }
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->smtpService = $smtpService;
        $this->smtpParams = $smtpParams;
        
        set_time_limit( 0 ); // Limite de tempo de execução: 2h. Deixe 0 (zero) para sem limite
        ignore_user_abort( true );
    }
    
    /**
     *  Atualiza o relacionamento de determinada Chave de Acesso a uma Autorização de Fornecimento.
     * 
     *  @param string $nrAcessoNfe  - Chave de acesso 
     *  @param $numeroAF            - Numero da AF
     *  @param $nrseqItProd         - Sequencial do item
     * 
     *  return boolean - Sucesso da atualização
     */
    public function updateNumeroAfXml($nrAcessoNfe, $numeroAf, $nrSeqItProd) {
        if(!$nrSeqItProd) {
            return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaNrAutoforn($numeroAf, $nrAcessoNfe, $nrSeqItProd);
        }
        else {
            return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaNrAutofornProd($numeroAf, $nrAcessoNfe, $nrSeqItProd);
        }
    } 
    
    /**
     *  Gera uma Nota Fiscal para determinada chave de acesso
     * 
     *  @param string $nrAcessoNfe  - Chave de acesso 
     *  @param string $cdTipoOper   - Tipo de operação da nota fiscal
     * 
     *  return array - Erros na geração da nota
     */
    public function gerarNfXml($nrAcessoNfe, $cdTipoOper) {
        $paramGeral = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getParamGeral($nrAcessoNfe, $cdTipoOper)[0];
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::MSG_ERROS["PROCEDURE_GERA_NF"]);
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, $nrAcessoNfe,                       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_GERAIGUALXML',    Param::PARAM_INPUT, $paramGeral["IDGERNFIGUALXML"],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdTipoOper,                        Param::PARAM_TYPE_STR));
        $procedure->execute();
        
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaNotaGerada($nrAcessoNfe);
    }
    
    /**
     *  Retorna o painel de inconsistencias de uma determinada chave de acesso
     * 
     *  @param string $nrAcessoNfe  - Chave de acesso 
     * 
     *  return array - Inconsistencias
     */
    public function getPainelInconsistencias($nrAcessoNfe) {
       return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getPainelInconsistencias($nrAcessoNfe);
    }
     
    /**
     *  Retorna os itens inconsistentes do painel de uma chave de acesso
     * 
     *  @param string $nrAcessoNfe  - Chave de acesso 
     *  @param string $cdPainel     - Código do painel
     * 
     *  return array - Inconsistencias
     */
    public function getItensInconsistencias($nrAcessoNfe, $cdPainel) {
       return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getItensInconsistencias($nrAcessoNfe, $cdPainel);
    }

    /**
     * 
     * 
     */
     
     public function getFilial($cdempresa, $nrinsjurfili){
         return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getFilial($nrinsjurfili, $cdempresa);
     }

    /**
     *  Retorna o arquivo xml de uma determinada chave de acesso
     * 
     *  @param string $nrAcessoNfe  - Chave de acesso  
     * 
     *  return string $xml - Xml referente a chave de acesso
     */
    public function getXml($nrAcessoNfe){
        $nFe = '';
        $protNFe = '';
        $params = array(
            "P_NRACESSONFE" => $nrAcessoNfe,
            "P_NRORG"       => $this->environment->getNrOrg()
        );
        
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getXml($params);
        
        foreach($retorno as $campo){
            if($campo["IDTIPOARQXML"] === 'D'){
                $nFe = $campo["DSARQXMLNFE_CLOB"];
            } else if($campo["IDTIPOARQXML"] === 'A'){
                $protNFe = $campo["DSARQXMLNFE_CLOB"];
            }
        }
        
        //Remove tags XML
        $nFe = preg_replace('/<\?xml[^>]+\>/', '', $nFe);
        $protNFe = preg_replace('/<\?xml[^>]+\>/', '', $protNFe);
        
        if(!$nFe || !$protNFe){
            throw new \Exception("Não foi possível gerar o Danfe");
        }
        
        $dom = new \DOMDocument('1.0', 'UTF-8');
        $dom->preservWhiteSpace = false;
        $dom->formatOutput      = false;
        $dom->loadXML($protNFe);
        
        $versao = $dom->getElementsByTagName('protNFe')->item(0)->getAttribute('versao');
        
        $xml = '<?xml version="1.0" encoding="UTF-8"?><nfeProc versao="' . $versao . '" xmlns="http://www.portalfiscal.inf.br/nfe">';
        
        //Verifica se $protNFe contém as informações da NFe e, caso verdadeiro, não utiliza $nFe. Isso evita que os itens sejam duplicados no documento.
        if ($dom->getElementsByTagName('NFe')->length) {
            $xml .= $protNFe . '</nfeProc>';
        }
        else {
            $xml .= $nFe . $protNFe . '</nfeProc>'; 
        }
        
        return $xml;
    }
    
    /**
     *  Gera o danfe a partir de um arquivo xml
     * 
     *  @param string $nracessonfe e   $cdfilial
     * 
     *  return string $arqDanfe - Diretorio do Danfe
     */
     
    public function gerarDanfe($xml, $nracessonfe){
        $caminho = sys_get_temp_dir() . DIRECTORY_SEPARATOR;
        $arqDanfe = $caminho . md5(uniqid()) . '.pdf';
        $orientacao = 'P';
        $papel = 'A4';
        $dirPdf = 'I';
        $dirLogo = '';
    
        try{
            
            //if($this->SAAS) {
            /*
            // ESTÁ COMENTADO POIS COMO A NOTA É DE ENTRADA PEGAR A LOGO DA FILIAL NÃO FAZ SENTIDO.
            $logo = isset($this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDiretorioLogo($nracessonfe)[0]["DSDIRLOGOMARCA"]) ? ($this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDiretorioLogo($nracessonfe)[0]["DSDIRLOGOMARCA"]) : '';

            if($logo) {
                $logo = file_get_contents( self::FILESERVER_LINK.strip_tags($logo));
                $dirLogo = sys_get_temp_dir().'/'.md5(uniqid()).'.jpeg';
                
                 if(is_file($dirLogo)) {
                    unlink($dirLogo);
                } 
            
                file_put_contents($dirLogo, $logo);
            } 
            */
            
            $danfe = new Danfe($xml, $orientacao, $papel,  '', $dirPdf);
            $danfe->monta();
            $pdf = $danfe->render();
        
            file_put_contents($arqDanfe, $pdf);
            
    
            /*    
            } else {
               
                $dirLogo = $caminho.$nracessonfe.".jpg";
                $logo = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getLogo($nracessonfe)[0]["DSARQLOGOEMPCLOB"];
                
                if(is_file($dirLogo)) {
                    unlink($dirLogo);
                } 
                
                if($logo != null) {
                    $file = imagecreatefromstring(base64_decode($logo));
                    imagejpeg($file, $dirLogo);
                }
                
                $danfe = new Danfe($xml, $orientacao, $papel,  $logo, $dirPdf);
                $pdf = $danfe->render();
                file_put_contents($arqDanfe, $pdf);
        
            }
            
            
            //Método montaDANFE não é mais chamado pois causa problemas com a nova versão da sped-da
            //$danfe->montaDANFE();
            //$danfe->printDocument($arqDanfe, 'F');
         */
        } catch(\Exception $e){
            throw new \Exception("Erro ao gerar o Danfe");
        }
       
        return $arqDanfe;
    }
    
    /**
    *   Metodo que realiza o upload de XML de NF-e para o sistema
    */
    
    public function uploadNotas($notas, $useVpd) {
        $diretorio = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDiretorio();
        
        if(!$useVpd && $diretorio[0]["DSDIRIMPARQXML"] == null){
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->updateDSDIRIMPARQXML_default();
            $diretorio = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDiretorio();
        }
            
        if(isset($diretorio[0]["DSDIRIMPARQXML"])) {
            
            $diretorio = $diretorio[0]["DSDIRIMPARQXML"];
            $tam = strlen($diretorio);
            if(substr($diretorio, $tam-1, $tam) !== "/")
                $diretorio = $diretorio."/";
            
            $this->diretorioImportacao = $diretorio;
            $this->criaDiretorios();
            
        }

        $impostos = array("ICMS" => '', "IPI" => '', "ISSQN" => '', "PIS" => '',
                          "COFINS" => '', "II" => '', "STICMS" => '', 
                          "STPIS" => '', "STCOFINS" => '');    
                          
        $this->verificaAssociaImpostos($impostos);
        
        $encargos = array("Frete" => '', "Seguro" => '', 
                              "Descontos" => '', "Outras Despesas" => '');
                              
        $encargos2 = $this->verificaAssociaEncargos($encargos);
        
        $paramIntegracao = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getParamIntegracao();
        $contImp = 0; $contErro = 0;
        
        foreach($notas as $nota) {
            $nomeArquivo = $nota['name'];
            $base64File = ltrim($nota['b64File'],'data:text/xml;base64,');
            $arquivo = base64_decode($base64File);

            try {
                $this->connection->beginTransaction();
                $xmlNotaFiscal  = new \DOMDocument("1.0", "UTF-8");
                try{
        		    $xmlNotaFiscal->loadXML($arquivo, LIBXML_NOBLANKS);
                }
                catch(\Exception $e){
                    throw new \Exception(self::ERROS_IMPORTACAO["ARQ_INV"]);
                }
    		    
                $arrayXml  = $this->converteXmlParaArray($arquivo);
                $isCTe     = ($xmlNotaFiscal->getElementsByTagName('cteProc')->length > 0);
                
                if(!$isCTe) {
                    $versaoXml = $arrayXml["@attributes"]["versao"];
                
                    if($versaoXml == "3.10" || $versaoXml == "4.00") {
                        $this->importacaoXML($nomeArquivo, $arrayXml, $arquivo, $this->environment->getCdOperador(), $contImp, $contErro, $impostos, $encargos, $paramIntegracao);
                    }
        		    else {
        		        throw new \Exception("Versão do XML não suportada pelo sistema.");
        		    }
                
                    $this->connection->commit();
                }
                else {
                    $nrAcessoNfe = substr($arrayXml["CTe"]["infCte"]["@attributes"]["Id"], 3);
                            
                    if($this->SAAS == false) {
                        $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                    }
                    else {
                        $nomeXml = $nrAcessoNfe.'.xml';
                    }
                    
                    $this->importaCTe($arrayXml, $xmlNotaFiscal, $nomeXml, $nomeArquivo, $arquivo);
                    $contImp++;
                    
                    $this->connection->commit();
                    
                    
                    //$this->connection->rollback();
                }
                
            }
            catch(\Exception $e) {
                $this->connection->rollback();
                $contErro++;
                throw new \Exception("Erro em " . $nomeArquivo . ": " . $e->getMessage());
            }
        }
        
        return array("ERROS" => $contErro, "SUCESSOS" => $contImp);
    }
    
    
    /**
    *   Metódo que importa NF-e de diretório/fileserver para o sistema.
    */
    public function importarNotas() {
        try {
            $this->erros = [];
            $nomeArquivos = $this->buscaNomeArquivosNotasImportacao();
            
            $impostos = array("ICMS" => '', "IPI" => '', "ISSQN" => '', "PIS" => '',
                              "COFINS" => '', "II" => '', "STICMS" => '', 
                              "STPIS" => '', "STCOFINS" => '');    
                              
            $this->verificaAssociaImpostos($impostos);
            
            $encargos = array("Frete" => '', "Seguro" => '', 
                              "Descontos" => '', "Outras Despesas" => '');
            
            $this->verificaAssociaEncargos($encargos);
            
            $paramIntegracao = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getParamIntegracao();
            $contImp = 0; $contErro = 0;
            
            foreach($nomeArquivos as $nomeArquivo) {
                $arquivo = file_get_contents($nomeArquivo);
                try {
                    $this->connection->beginTransaction();
                    $xmlNotaFiscal  = new \DOMDocument("1.0", "UTF-8");
                    try{
            		    $xmlNotaFiscal->loadXML($arquivo, LIBXML_NOBLANKS);
                    }
                    catch(\Exception $e){
                        throw new \Exception(self::ERROS_IMPORTACAO["ARQ_INV"]);
                    }
        		    
                    $arrayXml  = $this->converteXmlParaArray($arquivo);
                    $isCTe     = ($xmlNotaFiscal->getElementsByTagName('cteProc')->length > 0);
                    
                    if(!$isCTe) {
                        $versaoXml = $arrayXml["@attributes"]["versao"];
                    
                        if($versaoXml == "3.10" || $versaoXml == "4.00") {
                            $this->importacaoXML($nomeArquivo, $arrayXml, $arquivo, $this->environment->getCdOperador(), $contImp, $contErro, $impostos, $encargos, $paramIntegracao);
                        }
            		    else {
            		        throw new \Exception("Versão do XML não suportada pelo sistema.");
            		    }
                    
                        $this->connection->commit();
                    }
                    else {
                        $this->connection->rollback();
                    }
                    
                }
                catch(\Exception $e) {
                    $this->connection->rollback();
                    $contErro++;
                    // Depedendo do erro insere o log do erro na tabela LOGINCONSXML
                    $this->insereLogNota($e->getMessage(), $nomeArquivo);
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
        
        return array("ERROS" => $contErro, "SUCESSOS" => $contImp);
    }
    
    /**
    *   Metódo que importa CT-e de diretório/fileserver para o sistema.
    */
    public function importarCTe() {
        $this->erros = []; $contImp = 0; $contErro = 0;
        try {
            $nomeArquivos = $this->buscaNomeArquivosNotasImportacao();
            
            foreach($nomeArquivos as $nomeArquivo) {
                
                $arquivo = file_get_contents($nomeArquivo);
                try {
                    $this->connection->beginTransaction();
                    $xmlCTe  = new \DOMDocument("1.0", "UTF-8");
        		    $xmlCTe->loadXML($arquivo, LIBXML_NOBLANKS);
        		    
                    $arrayXml  = $this->converteXmlParaArray($arquivo);
                    $isCTe     = ($xmlCTe->getElementsByTagName('cteProc')->length > 0);
                    
                    if($isCTe) {
                        $nrAcessoNfe = substr($arrayXml["CTe"]["infCte"]["@attributes"]["Id"], 3);
                                
                        if($this->SAAS == false) {
                            $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                        }
                        else {
                            $nomeXml = $nrAcessoNfe.'.xml';
                        }
                        
                        $this->importaCTe($arrayXml, $xmlCTe, $nomeXml, $nomeArquivo);
                        $contImp++;
                    }
                    $this->connection->commit();
                }
                catch(\Exception $e) {
                    $this->connection->rollback();
                    $contErro++;
                    // Depedendo do erro insere o log do erro na tabela LOGINCONSXML
                    $this->insereLogCTe($e->getMessage(), $nomeArquivo);
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
   
        return array("SUCESSOS" => $contImp, "ERROS" => $contErro);
    }
    
    /**
     *  Importa a facturacion electrônica ( Nota Fiscal Colombiana )
     */
    public function importarFacturacionElectronica() {        
        $contImp = 0;
        $contErro = 0;
        try {
            
            $impostosFacturacion = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getImpostosFacturacion();
            $impostosFacturacion = $this->validaImpostosFacturacion($impostosFacturacion);
            
            $nomeArquivos = $this->buscaNomeArquivosNotasImportacao('xlsx');
            $notas = array();    

            foreach($nomeArquivos as $nomeArquivo) {
                $pathInfo = pathinfo($nomeArquivo);
                $extensao = $pathInfo["extension"];
                if($extensao == 'xlsx' || $extensao == 'XLSX') {
                    $notas = $this->lerFacturacionXLSX($nomeArquivo);
                }
                /*
                else if($extensao == 'txt' || $extensao == 'TXT') {
                    $notas = $this->lerFacturacionTXT($nomeArquivo);
                }
                */
                
               $totalProdutos = array();
   
                foreach($notas as $nota) {
                    if(isset($totalProdutos[$nota["CFD_campo_texto_20"]])) {
                        $totalProdutos[$nota["CFD_campo_texto_20"]]["total"] += $nota["DeclaredValue_COP"];
                    }
                    else {
                        $totalProdutos[$nota["CFD_campo_texto_20"]] = array(
                            "total" =>  (float) $nota["DeclaredValue_COP"]
                        );
                    }
                }
                
                try {
                    $notasImportadas    = array();
                    $notasErros         = array();
                    $importadosArq      = 0;
                    foreach($notas as $nota) {
                        try {
                            if(!isset($notasErros["CFD_campo_texto_20"])) {
                                $this->connection->beginTransaction();
                                $this->importarFacturacionColombia($nomeArquivo, $nota, $this->environment->getCdOperador(), $totalProdutos[$nota["CFD_campo_texto_20"]]["total"], $contImp, $contErro, $notasImportadas, $impostosFacturacion);
                                $this->connection->commit();
                                // $this->connection->rollback();
                                $importadosArq ++;
                            }
                        }
                        catch(\Exception $e) {
                            $this->connection->rollback();
                            $contErro++;
                            $notasErros[$nota["CFD_campo_texto_20"]] = true;
                            $this->insereLogFacturacion($e->getMessage(), $nomeArquivo, $nota);
                        }
                    }

                    if($importadosArq == count($notas)) {
                        if(!$this->SAAS) {
                            if(!is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."importados".DIRECTORY_SEPARATOR.date('Y-m'))) {
                                mkdir($this->diretorioImportacao.DIRECTORY_SEPARATOR."importados".DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                            }
                        }
                        $this->moveArquivoImportacao("importados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nomeArquivo);
                    }
                }
                catch(\Exception $e) {
                    var_dump($e->getMessage());
                }
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
        
        return array("SUCESSOS" => $contImp, "ERROS" => $contErro);
    }
     /**
      * Desvincula AF de uma Nota fiscal
     */
    public function updateDesvinculaAfNotafiscal($nrAcessoNfe, $numeroAf){
        
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->updateDesvinculaAfNotafiscal($nrAcessoNfe, $numeroAf);
    
    }
    public function desvinculaAfNotafiscal($nrAcessoNfe){
        
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->desvinculaAfNotaFiscal($nrAcessoNfe);
    
    }
    
    public function getInscEstadual($cdfilial){
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getInscEstadual($cdfilial);
    }
    
    /**
    * Desvincula AF por produto
    */
    public function updateDesvinculaAfProduto($nrAcessoNfe,$numeroAf, $nrSeqItProd){
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory-> updatedesvinculaAfProduto($nrAcessoNfe, $numeroAf, $nrSeqItProd);

    }
    
    public function desvinculaAfProduto($nrAcessoNfe, $nrSeqItProd){
        
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->desvinculaAfProduto($nrAcessoNfe, $nrSeqItProd);
    
    }
    
    
    
    /***************************************************************************
    *            Bloco de Funções referentes a importação de NF-e              *
    ****************************************************************************/
    /**
    *   Importa o xml da nota da versão 3.10 e 4.00.
    *   
    *   @param \DomDocument $xmlNota            - Xml da Nota Fiscal
    *   @param string $stringNota               - String da Nota Fiscal a ser importada
    *   @param string $operador                 - Código do operador que fez a operação
    *   @param int    &$contImp                 - Contador de notas fiscais importadas com sucesso
    *   @param int    &$contErro                - Contador de erros
    *   @param array  $impostos                 - Array com codigo dos impostos
    *   @param array  $encargos                 - Array com codigo dos encargos
    *   @param array  $paramIntegracao          - Parametros de integração
    */
    protected function importacaoXML($nomeArquivo, $arrayXml, $stringNota, $operador, &$contImp, &$contErro, $impostos, $encargos, $paramIntegracao) {
        
        $attFiliRece = true;
        if(isset($arrayXml["infNFeSupl"])) {
            unset($arrayXml["infNFeSupl"]);
        }
        
        $ide           = $arrayXml["NFe"]["infNFe"]["ide"];
	    $nrAcessoNfe   = substr($arrayXml["NFe"]["infNFe"]["@attributes"]["Id"], 3);
	    $emitente      = $arrayXml["NFe"]["infNFe"]["emit"];
	    $destinatario  = $arrayXml["NFe"]["infNFe"]["dest"];
	    $icmsTot       = $arrayXml["NFe"]["infNFe"]["total"]["ICMSTot"];
	    $organizacao   = $this->environment->getNrOrg();
	    $destinatario["IE"] = isset($destinatario["IE"]) ? $destinatario["IE"] : $this->getInscrEstad($destinatario["CNPJ"]);
	    // Verifica se a nota já foi importada
        if($this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaNotaImportada($nrAcessoNfe))
                throw new \Exception(self::ERROS_IMPORTACAO["ERR_INS"]);
        
        $this->validacoesImportacao($nrAcessoNfe, $ide, $destinatario, $emitente);
        $this->insereIntgNotaFiscal($nomeArquivo, $arrayXml, $ide, $emitente, $destinatario, $nrAcessoNfe, $attFiliRece, $paramIntegracao);
        
        if($ide["finNFe"] == '3' || $ide["finNFe"] == '4') {
            $geraNfsImp = false;
        }
        else {
            $geraNfsImp = true;
        }
        
        // Se for uma nota fiscal de estorno altera o status da nf do fornecedor para estornada quando possível.
        if($ide["finNFe"] == '3') {
            // Testar amanha
            if(isset($ide["NFref"]))  {
                $this->atualizaNotaEstornada($ide["NFref"][0]["refNFe"]);
                if($notaImportacao[0]["IDSTATUSNFE"] == "E") {
                    $this->atualizaNotaEstornada($nrAcessoNfe);
                }
            }

        }
        
        $ftValorDescItem = 0;
        $ftValorDesc = 0;
        $detProd = isset($arrayXml["NFe"]["infNFe"]["det"]["prod"]) ? array( 0 => $arrayXml["NFe"]["infNFe"]["det"]) : $arrayXml["NFe"]["infNFe"]["det"];

        foreach ($detProd as $contadorProduto => $det) {
                    
            if(!empty($det["VDesc"])){
                $ftValorDescItem += $det["VDesc"];
            }else{
                $ftValorDescItem += 0;
            }
               
        }
         $ftValorDesc = $ftValorDesc - $ftValorDescItem;
            
        // Insere os encargos da nota
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemEncargo('', '', $encargos["Frete"], $icmsTot["vFrete"], $nrAcessoNfe, $organizacao);
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemEncargo('', '', $encargos["Seguro"], $icmsTot["vSeg"], $nrAcessoNfe, $organizacao);
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemEncargo('', '', $encargos["Descontos"], $ftValorDesc, $nrAcessoNfe, $organizacao);
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemEncargo('', '', $encargos["Outras Despesas"], $icmsTot["vOutro"], $nrAcessoNfe, $organizacao);
        
        // Insere conta(s) a pagar
        $contadorContaPagar = 0;
        if(isset($arrayXml["NFe"]["infNFe"]["cobr"])) {
            $cobr = $arrayXml["NFe"]["infNFe"]["cobr"];
            
            if(isset($cobr["dup"][0])) {
                $umaConta = false;
            }
            else {
                $umaConta = true;
            }
            
            foreach ($cobr as $key => $value) {
                if($key === "dup" && $umaConta == true) {
                    $dVenc = isset($value["dVenc"]) ? $value["dVenc"] : '';
                    if($dVenc) $dVenc = $this->formataData($value["dVenc"]);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgContaPagar('', '', ($contadorContaPagar + 1), '00', '', '', 
                                                                        $dVenc, $value["vDup"], '', $nrAcessoNfe, $organizacao);
                }
            }
        }
        
        $vDesc = (float) $icmsTot["vDesc"];
        
        // Trecho de código manipula a string na mão pois <PED>XXXX</PED> não 
        // é reconhecido na leitura do XML.
        if(isset($arrayXml["NFe"]["infNFe"]["infAdic"])) {
            if(isset($arrayXml["NFe"]["infNFe"]["infAdic"]["infCpl"])) {
    		    $posInfCplInicial = strpos($stringNota, "<infCpl>") + 9;
    		    $posInfCplFinal   = strpos($stringNota, "</infCpl>");
    	
    		    $infCpl = substr($stringNota, $posInfCplInicial, ($posInfCplFinal - $posInfCplInicial));
    		    
                $posPedidoInicial = strpos(strtoupper($infCpl), "<PED>");
                if($posPedidoInicial != false) {
                    $posPedidoInicial += 5;
                    $posPedidoFinal = strpos(strtoupper($infCpl), "</PED>");
                    
                    if($posPedidoFinal != false) {
                        $cdPedido = substr($infCpl, $posPedidoInicial, ($posPedidoFinal - $posPedidoInicial));
                    }
                }
            }
        }
        
        if(!isset($cdPedido)) $cdPedido = '';
        // VERIFICAR
        $detProd = isset($arrayXml["NFe"]["infNFe"]["det"]["prod"]) ? array( 0 => $arrayXml["NFe"]["infNFe"]["det"]) : $arrayXml["NFe"]["infNFe"]["det"];
        foreach ($detProd as $contadorProduto => $det) {
            $this->insereProduto($det, $contadorProduto, $cdPedido, $nrAcessoNfe, $vDesc, $attFiliRece, $paramIntegracao, $impostos, $encargos);
        }
        $this->gravaHistoricoXML($nrAcessoNfe, 'DFE51000', $this->environment->getCdOperador(), '', '');

        if(isset($paramIntegracao[0])) {
            if($paramIntegracao[0]["IDIMPORTXMLNFE"] == 'S') {
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgArqXML($nrAcessoNfe, 'D', $this->removerAcentos($stringNota), $this->environment->getNrOrgTrab());
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgArqXML($nrAcessoNfe, 'A', $this->removerAcentos($stringNota), $this->environment->getNrOrgTrab());
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->setArqImportado($nrAcessoNfe);
            }
        }
        
        $paramManifesta = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getParamManifesta();
        
        if(isset($paramManifesta[0])) {
            if($paramManifesta[0]["IDCIENCIAEMISS"]  == '1') $idManif = '4';
            if($paramManifesta[0]["IDCONFIRMAEMISS"] == '1') $idManif = '1';
            
            $nota = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->buscaNotaFiscalImportada($nrAcessoNfe);
            
            if(isset($nota[0])) {
                $nota = $nota[0];
                $manifesta = false;
                $notaDestinada = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getNotaDestinada($nrAcessoNfe);
                if(isset($notaDestinada[0])) {
                    if(($notaDestinada[0]["IDSTATUSMANIF"] == 0 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 4 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 5 ) && $notaDestinada[0]["IDSTATUSMANIF"] != $idManif) {
                        $manifesta = true;
                    }
                }
                else {
                    $manifesta = true;
                }
                
                if($manifesta === true) {
                    // Manifesta nota
                    try {
                        $nrOrg   = $this->environment->getNrOrg();
                        $empresa = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getEmpresaPorCnpj($destinatario["CNPJ"]);
             
                        
                        if(isset($empresa[0])) {
                            $cdEmpresa = $empresa[0]["CDEMPRESA"];
                        }
                        else throw new \Exception("Não há empresa vinculada ao CNPJ da nota.");
                        
                        $config  = $this->manifestacaoService->montaConfigManifestacao($cdEmpresa, $nrOrg, $destinatario["IE"], '', true);
                        $nota    = array("NRACESSONFE" => $nrAcessoNfe, "NRNOTAFISC" => "", "NRLANCTONF" => "");
                        $retorno = $this->manifestacaoService->manifestarNfe(array($nota), $config, ($idManif == 4) ? "210210" : "210200", '');
                        if(isset($retorno["ERRO"])) throw new \Exception($retorno["ERRO"][0]["MOTIVO"]);
                    }
                    catch(\Exception $e) {
                        if($this->SAAS == false) {
                            $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                        }
                        else {
                            $nomeXml = $nrAcessoNfe.'.xml';
                        }
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota('Manifestação do destinatário',  substr($e->getMessage(), 1, 258), $nomeXml);
                    }
                }
            }
        }
        
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaNotaDestinada($nrAcessoNfe);

        if($this->SAAS == false) {
            $this->criaDiretorioSaveImportacao($destinatario);
            $name = explode($this->diretorioImportacao, $nomeArquivo);
            if (count($name) > 1) {
                $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
            }
            else {
                 $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[0];
            }
        }
        else {
            $nomeXml = $nrAcessoNfe.'.xml';
        }
        
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaNomeArquivo("NFe-$nrAcessoNfe.xml", $nrAcessoNfe);
        $contImp++;
        
        if($destinatario["IE"] != null) {
            $this->moveArquivoImportacao("importados".DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nrAcessoNfe, "NFe-$nrAcessoNfe.xml", $stringNota);
        }
        else {
            $this->moveArquivoImportacao("importados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nrAcessoNfe, "NFe-$nrAcessoNfe.xml", $stringNota);
        }
        
    }
    
    /**
    *   Insere a nota fiscal no sistema seguindo o modelo de NFe 3.10
    *   
    *   @param string  $nomeArquivo   - Nome do Arquivo xml
    *   @param array   $arrayXml      - Array com conteudo da Nota Fiscal
    *   @param array   $ide           - 
    *   @param array   $emitente      - Dados do emitente
    *   @param array   $destinatario  - Dados do destinatario
    *   @param int     $nrAcessoNfe   - Numero de Acesso da Nota Fiscal
    *   @param boolean $attFiliRece   - Atualiza Filial de recebimento
    *
    */
    private function insereIntgNotaFiscal($nomeArquivo, $arrayXml, $ide, $emitente, $destinatario, $nrAcessoNfe, &$attFiliRece, $paramIntegracao) {
        $nomeArquivo = str_replace($this->diretorioImportacao, "", $nomeArquivo);
        
        $infProt        = $arrayXml["protNFe"]["infProt"];
	    $totalProd      = $arrayXml["NFe"]["infNFe"]["total"]["ICMSTot"]["vProd"];
	    $totalNf        = $arrayXml["NFe"]["infNFe"]["total"]["ICMSTot"]["vNF"];
	    
        $cdFornecedor = $emitente["CPF"] ?? $emitente["CNPJ"] ?? '';
        $fornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getFornecedor($cdFornecedor)[0];
        
        $dtEmissao  = $this->formataData(substr($ide["dhEmi"], 0, 10));
        $dtEntrSaid = (isset($ide["dhSaiEnt"])) ? $this->formataData(substr($ide["dhSaiEnt"], 0, 10)) : '';
        
        $cdFilial   = '';
        $cdFiliRece = '';
        // Pega o código da filial caso esse seja informado no campo de infAdic
        if(isset($arrayXml["NFe"]["infNFe"]["infAdic"])) {
            $infAdic = $arrayXml["NFe"]["infNFe"]["infAdic"];
            foreach ($infAdic as $inf) {
                if(isset($inf["@attributes"])) {
                    if(isset($inf["@attributes"]["xCampo"])) {
                        if(strtoupper($inf["@attributes"]["xCampo"]) == "FILIAL") {
                            $cdFilial = ($inf["xTexto"]) ? substr($inf["xTexto"], 0, 4) : "";
                            $existeFilial = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaExisteFilial($cdFilial);
                            if($existeFilial == true) {
                                $cdFiliRece = $cdFilial;
                                $attFiliRece = false;
                            } 
                        }
                    }
                }
            }
        }
     
        $dhProtocolNfe = $this->formataData($infProt["dhRecbto"]);
        $nrInsJurEmpr  = (isset($destinatario["CNPJ"])) ? $destinatario["CNPJ"] : '';
        
        $nrInsJurTranxml = '';
        $nrPlacaVeicXml  = '';
        $sgUfVeicXml     = '';
        $qtDevoluXml     = '';
        $especVoluXml    = '';
        $marcaVoluXml    = '';
        $numeroVoluXml   = '';
        $vrPBrutoVoluXml = '';
        $vrPLiqVoluXml   = '';
        $idPgtoFreteXml  = '';
        
        if(isset($arrayXml["NFe"]["infNFe"]["transp"])) {
            $transp = $arrayXml["NFe"]["infNFe"]["transp"];
            if(isset($transp["transporta"])) {
                $transporta = $transp["transporta"];
                if(isset($transporta["CPF"])) {
                    $nrInsJurTranxml = $transporta["CPF"];
                }
                else if(isset($transporta["CNPJ"])) {
                    $nrInsJurTranxml = $transporta["CNPJ"];
                }
                else {
                    $nrInsJurTranxml = '';
                }
            }
            if(isset($transp["veicTransp"])) {
                if(isset($transp["veicTransp"]["placa"]))  $nrPlacaVeicXml = $transp["veicTransp"]["placa"];
                if(isset($transp["veicTransp"]["UF"]))     $sgUfVeicXml    = $transp["veicTransp"]["UF"];
            }
            if(isset($transp["vol"])) {
                if(isset($transp["vol"]["qVol"]))   $qtDevoluXml   = substr($transp["vol"]["qVol"], 1, 10);
                if(isset($transp["vol"]["esp"]))    $especVoluXml  = substr($transp["vol"]["esp"], 1, 10);
                if(isset($transp["vol"]["marca"]))  $marcaVoluXml  = $transp["vol"]["marca"];
                if(isset($transp["vol"]["nVol"]))   $numeroVoluXml = $transp["vol"]["nVol"];
                if(isset($transp["vol"]["pesoB"]))  $vrPBrutoVoluXml = $transp["vol"]["pesoB"];
                if(isset($transp["vol"]["pesoL"]))  $vrPLiqVoluXml   = $transp["vol"]["pesoL"];
            }
            
            if(isset($transp["modFrete"])) $idPgtoFreteXml = $transp["modFrete"];
        }
         
        
        $nomeXmlForne = ($this->SAAS == true) ? $nrAcessoNfe.'.xml' : $nomeArquivo;
        
        // Completar Parametros
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgNotaFiscal('', '', '', $destinatario["IE"], $fornecedor["IDTPIJURFORN"],
                $cdFornecedor, $fornecedor["CDFORNECED"], str_pad($ide["nNF"], 9, "0", STR_PAD_LEFT), $dtEmissao, $dtEntrSaid, $totalProd, $totalNf, $ide["serie"], $cdFiliRece, '', '', $nrAcessoNfe,
                'N', $dhProtocolNfe, $infProt["tpAmb"], $ide["tpEmis"], $infProt["nProt"], $infProt["xMotivo"], '55', $nomeXmlForne, $nrInsJurEmpr, 
                $nrInsJurTranxml, $nrPlacaVeicXml, $sgUfVeicXml,  $qtDevoluXml, $especVoluXml, $marcaVoluXml, $numeroVoluXml, $vrPBrutoVoluXml, $vrPLiqVoluXml, $idPgtoFreteXml, $this->environment->getNrOrgTrab());
    }
    
    /**
    *   Insere  os produtos da nota no sistema segundio o modelo de NFe 3.10 e 4.00
    *   
    *   @param array   $det                 - Dados Completos do produto (impostos, valores, etc)
    *   @param array   $contadorProduto     - Numero Sequencial do Produto inserido
    *   @param string  $cdPedido            - Numero do Pedido 
    *   @param array   $nrAcessoNfe         - Numero de acesso da nota fiscal
    *   @param array   $valorDescontoNota   - Valor total dos descontos na nota
    *   @param boolean $attFiliRece         - Atualiza Filial de recebimento
    *   @param array   $paramIntegracao     - Parametros para a integração
    *   @param array   $impostos            - Impostos com codigos associados
    *   @param array   $encargos            - Encargos com codigos associados
    */
    private function insereProduto($det, $contadorProduto, $cdPedido, $nrAcessoNfe, &$valorDescontoNota, &$attFiliRece, $paramIntegracao, $impostos, $encargos) {
        
        $prod = $det["prod"];
        
        if(!empty($prod["CFOP"])) {
            $tipoOper = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getTipoOperCfop("E", $prod["CFOP"]);
            if(isset($tipoOper[0])) {
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->updateTipoOperNf($tipoOper[0]["CDTIPOOPER"], $nrAcessoNfe);
            }
        }
        
        $nmProduto = substr($prod["xProd"], 0, 50);
        
        if(!$cdPedido) {
            if(isset($prod["xPed"])) {
                $nrAutoForn = str_pad($prod["xPed"], 10, "0", STR_PAD_LEFT);
                $autoForn   = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->buscaAutorizacaoFornecimento($nrAutoForn);
                if(!is_null($autoForn[0]["CDFILDEST"])) {
                    $cdFilial = $autoForn[0]["CDFILDEST"];
                    if($attFiliRece == true) {
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaFilialRecebimento($cdFilial, $nrAcessoNfe);
                    }
                    $attFiliRece = false;
                }
                else {
                    $nrAutoForn = '';
                }
            } 
            else {
                $nrAutoForn = $cdPedido;
            }
        }
        else $nrAutoForn = $cdPedido;
        if(isset($det["infAdProd"])) {
            if($attFiliRece == true) {
                $posFilial = strpos(strtoupper($det["infAdProd"]), 'FILIAL');
                if($posFilial !== false) {
                    if($posFilial != false) {
                        $posFilial += 6;
                        $cdFilial = substr(str_replace(" ", "", $det["infAdProd"]), $posFilial, ($posFilial + 4));
                        $existeFilial = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaExisteFilial($cdFilial);
                        if($existeFilial == true) {
                            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaFilialRecebimento($cdFilial, $nrAcessoNfe);
                        } 
                    }
                }
            }
        }
        
        if($attFiliRece == true) {
            if(isset($paramIntegracao[0])) {
                if($paramIntegracao[0]["NRFILPOSFINXML"] != 0) {
                    $posInicial = $paramIntegracao[0]["NRFILPOSINIXML"];
                    $posFinal = $paramIntegracao[0]["NRFILPOSFINXML"] - $paramIntegracao[0]["NRFILPOSINIXML"];
                    if(isset($det["infAdProd"])) {
                        $cdFilial = substr(str_replace(" ", "", $det["infAdProd"]), $posFilial, $posFinal);   
                    }
                    $existeFilial = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaExisteFilial($cdFilial);
                    if($existeFilial == true) {
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaFilialRecebimento($cdFilial, $nrAcessoNfe);
                    } 
                }
            }   
        }
        
        $cdInfAdProd = '';
        if(isset($paramIntegracao[0])) {
            if($paramIntegracao[0]["NRPROPOSFINXML"] != 0) {
                $posInicial = $paramIntegracao[0]["NRPROPOSINIXML"];
                $posFinal   = $paramIntegracao[0]["NRPROPOSFINXML"] - $paramIntegracao[0]["NRPROPOSINIXML"];
                if(isset($det["infAdProd"])) {
                    $cdProduto = substr(str_replace(".", "", $det["infAdProd"]), $posInicial, $posFinal);  
                    $produto = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->buscaProduto($cdProduto);
                    if(isset($produto[0])) {
                        $cdInfAdProd = $cdProduto;
                    } 
                }
            }
        }
        
        if(isset($prod["vDesc"])) {
            $descontoItem   = $prod["vDesc"];
        }
        else $descontoItem = 0;
        
        if(isset($prod["rastro"])) {
            $nLote = $prod["rastro"]["nLote"];
            $dFab = $this->formataData($prod["rastro"]["dFab"]);
            $dVal = $this->formataData($prod["rastro"]["dVal"]);
            $cAgreg = isset($prod["rastro"]["cAgreg"]) ? $prod["rastro"]["cAgreg"] : null;
        } else {
            $nLote = null;
            $dFab = null;
            $dVal = null;
            $cAgreg = null;
        }
        
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemProduto('', '', ($contadorProduto + 1), $prod["cProd"], $nmProduto, $prod["uCom"],
                                        $prod["qCom"], $prod["vUnCom"], $descontoItem, $prod["vProd"], '', '', $nrAutoForn, '', '', '', '', '', '', 
                                        $nrAcessoNfe, $cdInfAdProd, $prod["CFOP"], $prod["NCM"], $this->environment->getNrOrgTrab(), $nLote, $dFab,
                                        $dVal, $cAgreg);
        
        // Insere os Impostos do produto
        $this->insereImpostosProdutos($det, $nrAcessoNfe, $impostos, $contadorProduto);
    }
    
    /**
    *   Insere os impostos relacionados a um produto na tabela INTG_TRABITPROIMPOS 
    *   de acordo com o imposto.
    *   
    *   @param array   $det                 - Dados Completos do produto (impostos, valores, etc)
    *   @param array   $contadorProduto     - Numero Sequencial do Produto inserido
    *   @param array   $nrAcessoNfe         - Numero de acesso da nota fiscal
    *   @param array   $impostos            - Impostos com codigos associados
    */
    private function insereImpostosProdutos($det, $nrAcessoNfe, $impostos, $contadorProduto, $impostoProd = array()) {
        $organizacao = $this->environment->getNrOrgTrab();
        $impostoProd = empty($impostoProd) ? $det["imposto"] : $impostoProd;
        $orig = '';
        $cst  = '';
        
        // ICMS00
        if(isset($impostoProd["ICMSUFDest"]))
            $icmsUfDest = $impostoProd["ICMSUFDest"];
        if(isset($impostoProd["ICMS"]["ICMS00"])) {
            $icms00 = $impostoProd["ICMS"]["ICMS00"];
            
            $cdSitTribut = isset($icms00["orig"]) ? $icms00["orig"].$icms00["CST"] : null; //Grava null para CTe.
            
            $modBC  = (isset($icms00["modBC"]))  ? $icms00["modBC"]     : '';
            $vBC    = (isset($icms00["vBC"]))    ? $icms00["vBC"]       : '';
            $pICMS  = (isset($icms00["pICMS"]))  ? $icms00["pICMS"]     : 0;
            $vICMS  = (isset($icms00["vICMS"]))  ? $icms00["vICMS"]     : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms00["pFCP"]))   ? $icms00["pFCP"]      : 0;
            $vFcp   = (isset($icms00["vFCP"]))   ? $icms00["vFCP"]      : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, '', $pFcp, $vFcp, $organizacao);
            $orig = isset($icms00["orig"]) ? $icms00["orig"] : '';
            $cst  = $icms00["CST"];
        }
        
        // ICMS10
        else if(isset($impostoProd["ICMS"]["ICMS10"])) { // OK
            $icms10 = $impostoProd["ICMS"]["ICMS10"];
            $cdSitTribut = $icms10["orig"].$icms10["CST"];
            
            $vBC        = isset($icms10["vBC"])         ? $icms10["vBC"]        : '';
            $vBCST      = isset($icms10["vBCST"])       ? $icms10["vBCST"]      : '';
            $pICMS      = isset($icms10["pICMS"])       ? $icms10["pICMS"]      : 0 ;
            $pICMSST    = isset($icms10["pICMSST"])     ? $icms10["pICMSST"]    : 0 ;
            $vICMS      = isset($icms10["vICMS"])       ? $icms10["vICMS"]      : 0 ;
            $vICMSST    = isset($icms10["vICMSST"])     ? $icms10["vICMSST"]    : 0 ;
            $modBC      = isset($icms10["modBC"])       ? $icms10["modBC"]      : '';
            $modBCST    = isset($icms10["modBCST"])     ? $icms10["modBCST"]    : '';
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms10["pFCP"]))   ? $icms10["pFCP"]      : 0;
            $vFcp   = (isset($icms10["vFCP"]))   ? $icms10["vFCP"]      : 0;
            $pFcpST = (isset($icms10["pFCPST"])) ? $icms10["pFCPST"]    : 0;
            $vFcpST = (isset($icms10["vFCPST"])) ? $icms10["vFCPST"]    : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            if($pFcpST == 0){
                $pFcpST = (isset($icmsUfDest["pFCPUFDestST"])) ? $icmsUfDest["pFCPUFDestST"] : $pFcpST;
                $vFcpST = (isset($icmsUfDest["vFCPUFDestST"])) ? $icmsUfDest["vFCPUFDestST"] : $vFcpST;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, $modBCST, $pFcp, $vFcp, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', '', $vBCST, $pICMSST, $vICMSST,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', $pFcpST, $vFcpST, $organizacao);
            $orig = isset($icms10["orig"]) ? $icms10["orig"] : '';
            $cst  = $icms10["CST"];
        }

        // ICMS20
        else if(isset($impostoProd["ICMS"]["ICMS20"])) { // OK
            $icms20 = $impostoProd["ICMS"]["ICMS20"];
            $cdSitTribut = isset($icms20["orig"]) ? $icms20["orig"].$icms20["CST"] : null; //Grava null para CTe.
            
            $modBC  = (isset($icms20["modBC"]))  ? $icms20["modBC"]     : '';
            $pRedBC = (isset($icms20["pRedBC"])) ? $icms20["pRedBC"]    : '';
            $vBC    = (isset($icms20["vBC"]))    ? $icms20["vBC"]       : '';
            $pICMS  = (isset($icms20["pICMS"]))  ? $icms20["pICMS"]     : 0;
            $vICMS  = (isset($icms20["vICMS"]))  ? $icms20["vICMS"]     : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms20["pFCP"]))   ? $icms20["pFCP"]      : 0;
            $vFcp   = (isset($icms20["vFCP"]))   ? $icms20["vFCP"]      : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, '', $pFcp, $vFcp, $organizacao);
        }
        
        // ICMS30
        else if(isset($impostoProd["ICMS"]["ICMS30"])) { // ok
            $icms30 = $impostoProd["ICMS"]["ICMS30"];
            $cdSitTribut = $icms30["orig"].$icms30["CST"];
            $pRedBCST    = (isset($icms30["pRedBCST"])) ? $icms30["pRedBCST"]   : '';
            $vBCST      = isset($icms30["vBCST"])       ? $icms30["vBCST"]      : '';
            $pICMSST    = isset($icms30["pICMSST"])     ? $icms30["pICMSST"]    : 0 ;
            $vICMSST    = isset($icms30["vICMSST"])     ? $icms30["vICMSST"]    : 0 ;
            $modBCST    = isset($icms30["modBCST"])     ? $icms30["modBCST"]    : '';
            
             // FCP -> Somente no xml da versão 4.0
            $pFcpST = (isset($icms30["pFCPST"])) ? $icms30["pFCPST"]    : 0;
            $vFcpST = (isset($icms30["vFCPST"])) ? $icms30["vFCPST"]    : 0;
            
            if($pFcp == 0){
                $pFcpST = (isset($icmsUfDest["pFCPUFDestST"])) ? $icmsUfDest["pFCPUFDestST"] : $pFcpST;
                $vFcpST = (isset($icmsUfDest["vFCPUFDestST"])) ? $icmsUfDest["vFCPUFDestST"] : $vFcpST;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', $modBCST, $pFcpST, $vFcpST, $organizacao);
        
            $orig = isset($icms30["orig"]) ? $icms30["orig"] : '';
            $cst  = $icms30["CST"];
        }
        
        // ICMS40
        else if(isset($impostoProd["ICMS"]["ICMS40"])) { // OK
            $icms40 = $impostoProd["ICMS"]["ICMS40"];
            $cdSitTribut = $icms40["orig"].$icms40["CST"];
            if(isset($icms40["VICMSDeson"])) $valorICMS = (float) $icms40["VICMSDeson"];
            else $valorICMS = 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', 0, 0, $valorICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            $orig = isset($icms40["orig"]) ? $icms40["orig"] : '';
            $cst  = $icms40["CST"];
        }
        
        // ICMS51
        else if(isset($impostoProd["ICMS"]["ICMS51"])) { // OK
            $icms51 = $impostoProd["ICMS"]["ICMS51"];
            $cdSitTribut = $icms51["orig"].$icms51["CST"];
            $modBC  = (isset($icms51["modBC"]))  ? $icms51["modBC"]     : '';
            $pRedBC = (isset($icms51["pRedBC"])) ? $icms51["pRedBC"]    : '';
            $vBC    = (isset($icms51["vBC"]))    ? $icms51["vBC"]       : '';
            $pICMS  = (isset($icms51["pICMS"]))  ? $icms51["pICMS"]     : 0;
            $vICMS  = (isset($icms51["vICMS"]))  ? $icms51["vICMS"]     : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms51["pFCP"]))   ? $icms51["pFCP"]      : 0;
            $vFcp   = (isset($icms51["vFCP"]))   ? $icms51["vFCP"]      : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, '', $pFcp, $vFcp, $organizacao);
            $orig = isset($icms51["orig"]) ? $icms51["orig"] : '';
            $cst  = $icms51["CST"];
        }
        
        // ICMS60
        else if(isset($impostoProd["ICMS"]["ICMS60"])) { // OK
            $icms60 = $impostoProd["ICMS"]["ICMS60"];
            $cdSitTribut = isset($icms60["orig"]) ? $icms60["orig"].$icms60["CST"] : null; //Grava null para CTe.
            $vBCSTRet = isset($icms60["vBCSTRet"]) ? $icms60["vBCSTRet"] : '';
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms60["pFCPSTRet"]))   ? $icms60["pFCPSTRet"]      : 0;
            $vFcp   = (isset($icms60["vFCPSTRet"]))   ? $icms60["vFCPSTRet"]      : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', '', 0, 0,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', $pFcp, $vFcp, $organizacao);
            $orig = isset($icms60["orig"]) ? $icms60["orig"] : '';
            $cst  = $icms60["CST"];
        }
        
        // ICMS70
        else if(isset($impostoProd["ICMS"]["ICMS70"])) { // OK
            $icms70 = $impostoProd["ICMS"]["ICMS70"];
            $cdSitTribut = $icms70["orig"].$icms70["CST"];
            
            $pRedBC     = isset($icms70["pRedBC"])      ? $icms70["pRedBC"]     : '';
            $pRedBCST   = isset($icms70["pRedBCST"])    ? $icms70["pRedBCST"]   : '';
            $vBC        = isset($icms70["vBC"])         ? $icms70["vBC"]        : '';
            $vBCST      = isset($icms70["vBCST"])       ? $icms70["vBCST"]      : '';
            $pICMS      = isset($icms70["pICMS"])       ? $icms70["pICMS"]      : '';
            $pICMSST    = isset($icms70["pICMSST"])     ? $icms70["pICMSST"]    : '';
            $vICMS      = isset($icms70["vICMS"])       ? $icms70["vICMS"]      : '';
            $vICMSST    = isset($icms70["vICMSST"])     ? $icms70["vICMSST"]    : '';
            $modBC      = isset($icms70["modBC"])       ? $icms70["modBC"]      : '';
            $modBCST    = isset($icms70["modBCST"])     ? $icms70["modBCST"]    : '';
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms70["pFCP"]))   ? $icms70["pFCP"]      : 0;
            $vFcp   = (isset($icms70["vFCP"]))   ? $icms70["vFCP"]      : 0;
            $pFcpST = (isset($icms70["pFCPST"])) ? $icms70["pFCPST"]    : 0;
            $vFcpST = (isset($icms70["vFCPST"])) ? $icms70["vFCPST"]    : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            if($pFcp == 0){
                $pFcpST = (isset($icmsUfDest["pFCPUFDestST"])) ? $icmsUfDest["pFCPUFDestST"] : $pFcpST;
                $vFcpST = (isset($icmsUfDest["vFCPUFDestST"])) ? $icmsUfDest["vFCPUFDestST"] : $vFcpST;
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, $modBCST, $pFcp, $vFcp, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', $pFcpST, $vFcpST, $organizacao);
            $orig = isset($icms70["orig"]) ? $icms70["orig"] : '';
            $cst  = $icms70["CST"];
        }
        
        // ICMS90
        else if(isset($impostoProd["ICMS"]["ICMS90"])) { // OK
            $icms90     = $impostoProd["ICMS"]["ICMS90"];
            $cdSitTribut = isset($icms90["orig"]) ? $icms90["orig"].$icms90["CST"] : null; //Grava null para CTe.
            $pRedBC     = isset($icms90["pRedBC"])      ? $icms90["pRedBC"]     : '';
            $pRedBCST   = isset($icms90["pRedBCST"])    ? $icms90["pRedBCST"]   : '';
            $vBC        = isset($icms90["vBC"])         ? $icms90["vBC"]        : '';
            $vBCST      = isset($icms90["vBCST"])       ? $icms90["vBCST"]      : '';
            $pICMS      = isset($icms90["pICMS"])       ? $icms90["pICMS"]      : 0 ;
            $pICMSST    = isset($icms90["pICMSST"])     ? $icms90["pICMSST"]    : 0 ;
            $vICMS      = isset($icms90["vICMS"])       ? $icms90["vICMS"]      : 0 ;
            $vICMSST    = isset($icms90["vICMSST"])     ? $icms90["vICMSST"]    : 0 ;
            $modBC      = isset($icms90["modBC"])       ? $icms90["modBC"]      : '';
            $modBCST    = isset($icms90["modBCST"])     ? $icms90["modBCST"]    : '';
            
            // FCP -> Somente no xml da versão 4.0
            $pFcp   = (isset($icms90["pFCP"]))   ? $icms90["pFCP"]      : 0;
            $vFcp   = (isset($icms90["vFCP"]))   ? $icms90["vFCP"]      : 0;
            $pFcpST = (isset($icms90["pFCPST"])) ? $icms90["pFCPST"]    : 0;
            $vFcpST = (isset($icms90["vFCPST"])) ? $icms90["vFCPST"]    : 0;
            
            if($pFcp == 0){
                $pFcp = (isset($icmsUfDest["pFCPUFDest"])) ? $icmsUfDest["pFCPUFDest"] : $pFcp;
                $vFcp = (isset($icmsUfDest["vFCPUFDest"])) ? $icmsUfDest["vFCPUFDest"] : $vFcp;
            }
            if($pFcp == 0){
                $pFcpST = (isset($icmsUfDest["pFCPUFDestST"])) ? $icmsUfDest["pFCPUFDestST"] : $pFcpST;
                $vFcpST = (isset($icmsUfDest["vFCPUFDestST"])) ? $icmsUfDest["vFCPUFDestST"] : $vFcpST;
            }
            
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $modBC, $modBCST, $pFcp, $vFcp, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', $pFcpST, $vFcpST, $organizacao);
            $orig = isset($icms90["orig"]) ? $icms90["orig"] : '';
            $cst  = $icms90["CST"];
        }
        
        // ICMSPART
        else if(isset($impostoProd["ICMS"]["ICMSPart"])) { // OK
            $icmsPart = $impostoProd["ICMS"]["ICMSPart"];
            $cdSitTribut = $icmsPart["orig"].$icmsPart["CST"];
            $pRedBC = isset($icmsPart["pRedBC"]) ? $icmsPart["pRedBC"] : '';
            $pRedBCST = isset($icmsPart["pRedBCST"]) ? $icmsPart["pRedBCST"] : '';
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $icmsPart["vBC"], $icmsPart["pICMS"], $icmsPart["vICMS"],  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, $icmsPart["modBC"], $icmsPart["modBCST"], 0, 0, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $icmsPart["vBCST"], $icmsPart["pICMSST"], $icmsPart["vICMSST"],  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            $orig = isset($icmsPart["orig"]) ? $icmsPart["orig"] : '';
            $cst  = $icmsPart["CST"];
        }
        
        // ICMSST     - Não implementado
        
        // ICMSSN101
        else if(isset($impostoProd["ICMS"]["ICMSSN101"])) { // OK
            $icmsSN101 = $impostoProd["ICMS"]["ICMSSN101"];
            // $cdSitTribut = $icmsSN101["orig"].$icmsSN101["CST"];
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', 0, 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            $orig = isset($icmsSN101["orig"]) ? $icmsSN101["orig"] : '';
            $cst  = '';
        }
        
        // ICMSSN102
        else if(isset($impostoProd["ICMS"]["ICMSSN102"])) { // OK
            $icmsSN102 = $impostoProd["ICMS"]["ICMSSN102"];
            // $cdSitTribut = $icmsSN102["orig"].$icmsSN102["CST"];
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', 0, 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            $orig = isset($icmsSN102["orig"]) ? $icmsSN102["orig"] : '';
            $cst  = '';
        }
        
        // ICMSSN201
        else if(isset($impostoProd["ICMS"]["ICMSSN201"])) { // OK
            $icmsSN201 = $impostoProd["ICMS"]["ICMSSN201"];
            
            // $cdSitTribut = $icmsSN201["orig"].$icmsSN201["CST"];
            $modBCST = $icmsSN201["modBCST"];
            
            $pRedBCST = isset($icmsSN201["pRedBCST"]) ? $icmsSN201["pRedBCST"] : '';
            
            $vBCST = isset($icmsSN201["vBCST"]) ? $icmsSN201["vBCST"]    : '';
            
            $pICMSST = isset($icmsSN201["pICMSST"]) ? $icmsSN201["pICMSST"]    : 0;
            $vICMSST = isset($icmsSN201["vICMSST"]) ? $icmsSN201["vICMSST"]    : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcpST = isset($icmsSN201["pFCPST"]) ? $icmsSN201["pFCPST"]    : 0;
            $vFcpST = isset($icmsSN201["vFCPST"]) ? $icmsSN201["vFCPST"]    : 0;
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', '', 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', $modBCST, 0, 0, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', $modBCST, $pFcpST, $vFcpST, $organizacao);
                                                        
            $orig = isset($icmsSN201["orig"]) ? $icmsSN201["orig"] : '';
            $cst  = '';
        }
        
        // ICMSSN202
        else if(isset($impostoProd["ICMS"]["ICMSSN202"])) { // OK
            $icmsSN202 = $impostoProd["ICMS"]["ICMSSN202"];
            
            // $cdSitTribut = $icmsSN202["orig"].$icmsSN202["CST"];
            $modBCST = $icmsSN202["modBCST"];
            
            $pRedBCST = isset($icmsSN202["pRedBCST"]) ? $icmsSN202["pRedBCST"] : '';
            
            $vBCST = isset($icmsSN202["vBCST"]) ? $icmsSN202["vBCST"] : '';
            
            $pICMSST = isset($icmsSN202["pICMSST"]) ? $icmsSN202["pICMSST"] : 0;
            $vICMSST = isset($icmsSN202["vICMSST"]) ? $icmsSN202["vICMSST"] : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcpST = isset($icmsSN202["pFCPST"]) ? $icmsSN202["pFCPST"]    : 0;
            $vFcpST = isset($icmsSN202["vFCPST"]) ? $icmsSN202["vFCPST"]    : 0;
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', '', 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', $modBCST, 0, 0, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', $modBCST, $pFcpST, $vFcpST, $organizacao);
                                                        
            $orig = isset($icmsSN202["orig"]) ? $icmsSN202["orig"] : '';
            $cst  = '';
        }
        
        // ICMSSN500
        else if(isset($impostoProd["ICMS"]["ICMSSN500"])) { // OK
            $icmsSN500 = $impostoProd["ICMS"]["ICMSSN500"];
            // $cdSitTribut = $icmsSN500["orig"].$icmsSN500["CST"];
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', '', '', 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
                                                        
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', '', 0, 0, 0,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
                                                        
            $orig = isset($icmsSN500["orig"]) ? $icmsSN500["orig"] : '';
            $cst  = '';
        }
        
        // ICMSSN900
        else if(isset($impostoProd["ICMS"]["ICMSSN900"])) { // OK
            $icmsSN900 = $impostoProd["ICMS"]["ICMSSN900"];
            
            $cdSitTribut = $icmsSN900["CSOSN"];
            $modBC = $icmsSN900["modBC"] ?? '';
            
            $vBC = isset($icmsSN900["vBC"]) ? $icmsSN900["vBC"]           : '';
            $pRedBC = isset($icmsSN900["pRedBC"]) ? $icmsSN900["pRedBC"]  : '';
            
            $pICMS = isset($icmsSN900["pICMS"]) ? $icmsSN900["pICMS"]  : 0;
            $vICMS = isset($icmsSN900["vICMS"]) ? $icmsSN900["vICMS"]  : 0;
            
            $pRedBCST = isset($icmsSN900["pRedBCST"]) ? $icmsSN900["pRedBCST"]  : '';
            
            $vBCST = isset($icmsSN900["vBCST"]) ? $icmsSN900["vBCST"]  : '';
            
            $pICMSST = isset($icmsSN900["pICMSST"]) ? $icmsSN900["pICMSST"]  : 0;
            $vICMSST = isset($icmsSN900["vICMSST"]) ? $icmsSN900["vICMSST"]  : 0;
            
            // FCP -> Somente no xml da versão 4.0
            $pFcpST = isset($icmsSN900["pFCPST"]) ? $icmsSN900["pFCPST"]  : 0;
            $vFcpST = isset($icmsSN900["vFCPST"]) ? $icmsSN900["vFCPST"]  : 0;

            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ICMS"],
                                                        '', $pRedBC, $vBC, $pICMS, $vICMS,  '', 
                                                        $cdSitTribut, '', '1', $nrAcessoNfe, '', $modBC, 0, 0, $organizacao);
                                                     
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STICMS"],
                                                        '', $pRedBCST, $vBCST, $pICMSST, $vICMSST,  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', $pFcpST, $vFcpST, $organizacao);
            
            $orig = isset($icmsSN900["orig"]) ? $icmsSN900["orig"] : '';
            $cst  = '';                                      
        }
        
        if($orig !== '') {
            if($orig ==  1 || $orig == 2 || $orig == 6 || $orig == 7) {
                $origProd = 'E';
            }
            else {
                $origProd = 'N';
            }
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaOrigemProd($origProd, $nrAcessoNfe, ($contadorProduto + 1));
        }
        
        // FIM DA PARTE REFERENTE AO ICMS
        
        // IPI
        if(isset($impostoProd["IPI"])) { // OK
            if(isset($impostoProd["IPI"]["IPITrib"])) {
                $ipi        = $impostoProd["IPI"]["IPITrib"];
                $aliquota   = (isset($ipi["pIPI"]))     ? $ipi["pIPI"] : 0;
                $vBC        = (isset($ipi["vBC"]))      ? $ipi["vBC"]  : '';
                $cst        = (isset($ipi["CST"]))      ? $ipi["CST"]  : '';
                $vIPI       = (isset($ipi["vIPI"]))     ? $ipi["vIPI"] : 0;
                
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["IPI"],
                                                            '', '', $vBC, $aliquota, $vIPI,  '', 
                                                            $cst, '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
                                                         
                if($cst == '') {
                    $cst = $ipi["CST"];
                }
            }
        }
        
        // II
        if(isset($impostoProd["II"])) { // OK
            $ii = $impostoProd["II"];
         
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["II"],
                                                        '', '', $ii["vBC"], 0, $ii["vII"],  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
        }
        
        // INICIO DA PARTE REFERENTE AO PIS
        // PISAliq
        if(isset($impostoProd["PIS"]["PISAliq"])) { // OK
            $pisAliq = $impostoProd["PIS"]["PISAliq"];
            $aliquota = (isset($pisAliq["pPIS"])) ? $pisAliq["pPIS"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["PIS"],
                                                        '', '', $pisAliq["vBC"], $aliquota, $pisAliq["vPIS"],  '', 
                                                        $pisAliq["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $pisAliq["CST"];
            }
        }
        
        // PISQtde
        else if(isset($impostoProd["PIS"]["PISQtde"])) { // OK
            $pisQtde = $impostoProd["PIS"]["PISQtde"];
            $aliquota = (isset($pisQtde["vAliqProd"])) ? round((($pisQtde["vAliqProd"] * 100) / $det["prod"]["vProd"]), 2) : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["PIS"],
                                                        '', '', $det["prod"]["vProd"], $aliquota, $pisQtde["vPIS"],  '', 
                                                        $pisQtde["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $pisQtde["CST"];
            }
        }
        
        // PISNT
        else if(isset($impostoProd["PIS"]["PISNT"])) { // OK
            $pisNT = $impostoProd["PIS"]["PISNT"];
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["PIS"],
                                                        '', '', 0, 0, 0,  '', 
                                                        $pisNT["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
                                                        
            
            if($cst == '') {
                $cst = $pisNT["CST"];
            }
        }
        
        // PISOUTR
        else if(isset($impostoProd["PIS"]["PISOutr"])) { // OK
            $pisOutr = $impostoProd["PIS"]["PISOutr"];
            $vBc = isset($pisOutr["vBC"]) ? $pisOutr["vBC"] : 0;
            $aliquota = (isset($pisOutr["pPIS"])) ? $pisOutr["pPIS"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["PIS"],
                                                        '', '', $vBc, $aliquota, $pisOutr["vPIS"],  '', 
                                                        $pisOutr["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $pisOutr["CST"];
            }
        }
        
        //PISST
        else if(isset($impostoProd["PIS"]["PISST"])) { // OK
            $pisSt = $impostoProd["PIS"]["PISST"];
            $aliquota = (isset($pisSt["pPIS"])) ? $pisSt["pPIS"] : 0;
            $vBc = isset($pisSt["vBC"]) ? $pisSt["vBC"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STPIS"],
                                                        '', '', $vBc, $aliquota, $pisSt["vPIS"],  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $pisSt["CST"];
            }
        }
        
        // FIM    DA PARTE REFERENTE AO PIS
        // INICIO DA PARTE REFERENTE AO COFINS
        
        // COFINSAliq
        if(isset($impostoProd["COFINS"]["COFINSAliq"])) { // OK
            $cofinsAliq = $impostoProd["COFINS"]["COFINSAliq"];
            $aliquota = (isset($cofinsAliq["pCOFINS"])) ? $cofinsAliq["pCOFINS"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["COFINS"],
                                                            '', '', $cofinsAliq["vBC"], $aliquota, $cofinsAliq["vCOFINS"],  '', 
                                                            $cofinsAliq["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $cofinsAliq["CST"];
            }
        }
        
        // COFINSQtde
        else if(isset($impostoProd["COFINS"]["COFINSQtde"])) { // OK
            $cofinsQtde = $impostoProd["COFINS"]["COFINSQtde"];
            $aliquota = (isset($cofinsQtde["vAliqProd"])) ? round((($cofinsQtde["vAliqProd"] * 100) / $det["prod"]["vProd"]), 2) : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["COFINS"],
                                                        '', '', $det["prod"]["vProd"], $aliquota, $cofinsQtde["vCOFINS"],  '', 
                                                        $cofinsQtde["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $cofinsQtde["CST"];
            }
        }
        
        // COFINSNT
        else if(isset($impostoProd["COFINS"]["COFINSNT"])) { // OK
            $cofinsNT = $impostoProd["COFINS"]["COFINSNT"];
            
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["COFINS"],
                                                        '', '', 0, 0, 0,  '', 
                                                        $cofinsNT["CST"], '', '1', $nrAcessoNfe, '', '',  0, 0, $organizacao);
                                                        
            
            if($cst == '') {
                $cst = $cofinsNT["CST"];
            }
        }

        //COFINSOutr
        else if(isset($impostoProd["COFINS"]["COFINSOutr"])) { // OK
            $cofinsOutr = $impostoProd["COFINS"]["COFINSOutr"];
            $aliquota = (isset($cofinsOutr["pCOFINS"])) ? $cofinsOutr["pCOFINS"] : 0;
            $vBC = isset($cofinsOutr["vBC"]) ? $cofinsOutr["vBC"] : 0; 
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["COFINS"],
                                                        '', '', $vBC, $aliquota, $cofinsOutr["vCOFINS"],  '', 
                                                        $cofinsOutr["CST"], '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $cofinsOutr["CST"];
            }
        }
        
        //COFINSST
        else if(isset($impostoProd["COFINS"]["COFINSST"])) { // OK
            $cofinsSt = $impostoProd["COFINS"]["COFINSST"];
            $aliquota = (isset($cofinsSt["pCOFINS"])) ? $cofinsSt["pCOFINS"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["STCOFINS"],
                                                        '', '', $cofinsSt["vBC"], $aliquota, $cofinsSt["vCOFINS"],  '', 
                                                        '', '', '1', $nrAcessoNfe, '', '', 0, 0, $organizacao);
            
            if($cst == '') {
                $cst = $cofinsSt["CST"];
            }
        }
        
        // FIM DA PARTE REFERENTE AO COFINS
        
        // ISSQN
         if(isset($impostoProd["ISSQN"]["ISSQNAliq"])) {
            $ISSQNAliq = $impostoProd["ISSQN"]["ISSQN"];
            $aliquota = (isset($ISSQN["vAliq"])) ? $ISSQN["vAliq"] : 0;
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', ($contadorProduto + 1), $impostos["ISSQN"],
                                                            '', '', $ISSQN["vBC"], $aliquota, $ISSQN["vISSQN"],  '', 
                                                            $ISSQN["CST"], '', '1', $nrAcessoNfe, '', '',  0, 0, $organizacao);
        }
        
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->updateCST($cst, $nrAcessoNfe, ($contadorProduto + 1));
    }
    
    private function removeTagInfNfSupl($stringNota) {
        $posicaoInicial = strrpos($stringNota, "<infNFeSupl>");

        if ($posicaoInicial !== false) {
            $inicio     = explode("<infNFeSupl>", $stringNota)[0];
            $final      = explode("</infNFeSupl>", $stringNota)[1];
            $stringNota = $inicio.$final;
        }
        return $stringNota;
    }
    
    public function gravaHistoricoXML($nrAcessoNfe, $idMovimento, $operador, $cdFilial, $nrLanctonf = '', $idManif = '', $nrProtocolo = '') {
        $historicoXML = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->buscaHistoricoXML($nrAcessoNfe);
        if(!isset($historicoXML[0])) {
            if($idMovimento == 'DFE51000') {
                $sequencial = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getSequencialHistoricoXML($nrAcessoNfe)[0]["NRSEQHISTOR"];
                if($sequencial === null) $sequencial = 1;
                else $sequencial += 1;
                $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereHistoricoXML($nrAcessoNfe, $sequencial, date('d/m/Y H:i:s'), '1', 'N', '', self::IMPORTACAO_XML, $cdFilial, $nrLanctonf, $operador, $this->environment->getNrOrg());
            }
            else if($idMovimento == "DFE51001") {
                 $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereHistoricoXML($nrAcessoNfe, $sequencial, date('d/m/Y H:i:s'), '1', 'N', '', self::GERA_XML, $cdFilial, $nrLanctonf, $operador, $this->environment->getNrOrg());
            }
        }
    }
    
    /***************************************************************************
    *            Bloco de Funções referentes a importação de CT-e              *
    ****************************************************************************/
    
    /**
    *   Importa o CT-e.
    *   
    *   @param array  $arrayXml         - Array contendo os dados do arquivo CT-e
    *   @param \DomDocument $xmlCTe     - Xml do CT-e
    *   @param string $nomeXml          - Nome do arquivo XML
    *   @param string &$nomeArquivo     - Nome do arquivo a ser salvo em caso de sucesso
    */
    public function importaCte($arrayXml, $xmlCTe, $nomeXml, $nomeArquivo, $arquivo = '') {
        $infCTe = $arrayXml["CTe"]["infCte"];
        $ide    = $infCTe["ide"];
        $nrAcessoNFe = str_replace("CTe", "", $infCTe["@attributes"]["Id"]);
        
        $dhEmi = date("d/m/Y", strtotime(substr($xmlCTe->getElementsByTagName('dhEmi')->item(0)->nodeValue, 0, 10)));
        $nrNotaFisc = str_pad($xmlCTe->getElementsByTagName('nCT')->item(0)->nodeValue, 9, "0", STR_PAD_LEFT);

        $serie  = $xmlCTe->getElementsByTagName('serie')->item(0)->nodeValue;
        $tpEmis = $xmlCTe->getElementsByTagName('tpEmis')->item(0)->nodeValue;
        $cfop   = $xmlCTe->getElementsByTagName('CFOP')->item(0)->nodeValue;
        
        $toma = (isset($ide["toma3"])) ? $ide["toma3"]["toma"] : $ide["toma4"]["toma"];
        $tomador = isset($infCTe["rem"]) ? $infCTe["rem"] : null;
        
        if($tomador) {
            $inscTomador = isset($tomador["CNPJ"]) ? $tomador["CNPJ"] : $tomador["CPF"];
            $ieTomador   = isset($tomador["IE"]) ? $tomador["IE"] : null;
            $nomeTomador = substr($tomador["xNome"], 0, 40);
        }
        
        $munOrig     = $ide["cMunIni"];
        $munDest     = $ide["cMunFim"];
        
        switch($toma) {
            case 0:
                $dest = $infCTe["rem"];
                break;
            case 1:
                $dest = $infCTe["exped"];
                break;
            case 2:
                $dest = $infCTe["receb"];
                break;
            case 3:
                $dest = $infCTe["dest"];
                break;
            case 4:
                $dest = $ide["toma4"];
                break;
        }
        
        $cnpjDest = isset($dest["CNPJ"]) ? $dest["CNPJ"] : $dest["CPF"];
        $ieDest   = $dest["IE"] = isset($dest["IE"]) ? $dest["IE"] : $this->getInscrEstad($cnpjDest);
        $nomeDest = substr($dest["xNome"], 0, 40);
        
        if($this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaNotaImportada($nrAcessoNFe)) throw new \Exception(self::ERROS_IMPORTACAO["ERR_INS"]);
        
        $prest      = $infCTe["vPrest"];
        $vrPrest    = $prest["vTPrest"]; 
        
/*        
        $icms  = $infCTe["imp"]["ICMS"]["ICMS45"];
        $cst   = $icms["CST"];
        $vBc   = isset($icms["vBC"])   ? $icms["vBC"]   : '';
        $pICMS = isset($icms["pICMS"]) ? $icms["pICMS"] : '';
        $vICMS = isset($icms["vICMS"]) ? $icms["vICMS"] : '';*/
        
        $infProt    = $arrayXml["protCTe"]["infProt"];
        $dhRecbto   = $infProt["dhRecbto"];
        $tpAmb      = $infProt["tpAmb"];
        $nProt      = $infProt["nProt"];
        $xMotivo    = $infProt["xMotivo"];
        
        // Valida Fornecedor e série
        $emit     = $infCTe["emit"];
        $cnpjForn = isset($emit["CNPJ"]) ? $emit["CNPJ"] : $emit["CPF"];
        $ieForn   = $emit["IE"];
        $fornecedor = $this->validaFornecedorCTe($cnpjForn,$emit);
        $idTpInsjurForn = $fornecedor["IDTPIJURFORN"];
        $cdForneced = $fornecedor["CDFORNECED"];
        $this->validaSerie($serie);
        
        $dtHrProtocoloNfe = date("d/m/Y", strtotime($dhRecbto));
        
        // Insere CTe
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereNFCP('', '', '', $ieDest, $idTpInsjurForn, $cnpjForn, $cdForneced, $nrNotaFisc, $dhEmi, '', $vrPrest, $vrPrest, $serie, '', 
                                '', '', $nrAcessoNFe, 'N', $dtHrProtocoloNfe, $tpAmb, $tpEmis, $nProt, $xMotivo, '57', $nomeXml, '1', $this->environment->getNrOrgTrab(), $munOrig, $munDest);
        
        $this->gravaHistoricoXML($nrAcessoNFe, 'DFE51000', $this->environment->getCdOperador(), '', '');
        
        // Insere Item CTe
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemProduto('', '', 1, 'ITEM_CTE', 'FRETE', 'UN ',
                                        1, $vrPrest, 0, $vrPrest, '', '', '', '', '', '', '', '', '', 
                                        $nrAcessoNFe, '', $cfop, '', $this->environment->getNrOrgTrab());
                                                        
        $buscaImpostos = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getImpostos();
        $impostos = ['ICMS' => '', 'STICMS' => ''];
        
        if($buscaImpostos) {
            foreach ($buscaImpostos as $posicaoimposto) {
                if($posicaoimposto['SGIMPOSTONFE'] == 'ICMS')    $impostos['ICMS']   = $posicaoimposto['CDIMPOSTO'];
                if($posicaoimposto['SGIMPOSTONFE'] == 'ICMS ST') $impostos['STICMS'] = $posicaoimposto['CDIMPOSTO'];
            }
            
            foreach ($impostos as $nomeImposto => $cdImposto) {
                if(empty($cdImposto)) throw new \Exception(self::MSG_ERROS["IMP_ASO"]." Imposto $nomeImposto não associado.");
            }
        } else throw new \Exception(self::MSG_ERROS["IMP_ASO"]);
        
        $this->insereImpostosProdutos([], $nrAcessoNFe, $impostos, 0, $infCTe["imp"]);
        
        if(isset($infCTe["infCTeNorm"]["infDoc"])) {
            if(isset($infCTe["infCTeNorm"]["infDoc"]["infNFe"])) {
                $infNfe = $infCTe["infCTeNorm"]["infDoc"]["infNFe"];
                // Somente uma Nf-e relacionada
                if(isset($infNfe["chave"])) {
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereRelacCteNfe($infNfe["chave"], $nrAcessoNFe, $this->environment->getNrOrgTrab());
                }
                // Varias Nf-e
                else {
                    foreach ($infNfe as $nfe) {
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereRelacCteNfe($nfe["chave"], $nrAcessoNFe, $this->environment->getNrOrgTrab());
                    }
                }
            }
        }
        // Insere conta(s) a pagar
        if(isset($infCTe["infCTeNorm"]["cobr"])) {
            $contadorContaPagar = 0;
            $cobr = $infCTe["infCTeNorm"]["cobr"];
            
            $umaConta = isset($cobr["dup"][0]) ? false : true;
            
            foreach ($cobr as $key => $value) {
                if($key === "dup" && $umaConta == true) {
                    $dVenc = isset($value["dVenc"]) ? $value["dVenc"] : '';
                    if($dVenc) $dVenc = $this->formataData($value["dVenc"]);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgContaPagar('', '', ($contadorContaPagar + 1), '00', '', '', 
                                                                        $dVenc, $value["vDup"], '', $nrAcessoNFe, $this->environment->getNrOrgTrab());
                }
            }
        }
        
        if($this->SAAS == false) {
            $this->criaDiretorioSaveImportacao(array("IE" => $ieDest));
        }
        if($ieDest != null) {
            $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."importados".DIRECTORY_SEPARATOR.$ieDest.DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nrAcessoNFe, "NFe-$nrAcessoNFe.xml", $arquivo);
        }
        else {
            $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."importados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nrAcessoNFe, "NFe-$nrAcessoNFe.xml");
        }
    }
    
    public function atualizaCfopTpOp($cfop, $nrAcessoNfe) {
        $tipoOper = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getTpOperCfOp('E', $cfop);
        if(isset($tipoOper[0])) {
            $this->updateTipoOperNf($tipoOper[0]["CDTIPOOPER"], $nrAcessoNfe);
        }
    }
    
    /***************************************************************************
    *               Bloco de Funções de Validações                             *
    ****************************************************************************/
    
    /**
    *   Realiza as validações necessarias para a importação da nota fiscal
    *
    *   @param string $nrAcessoNfe       - Chave de Acesso da Nota Fiscal
    *   @param array $ide                - Dados para identificação da nota fiscal
    */
    protected function validacoesImportacao($nrAcessoNfe, $ide, $destinatario, $emitente) {
        $statusNFe = $this->validaStatusNFe($nrAcessoNfe, $ide["tpAmb"], $ide["tpEmis"], '', $destinatario["CNPJ"], $destinatario["IE"]);

        if($statusNFe == 1 || $statusNFe == 8) {
            $this->validaFornecedorEDestinatario($destinatario, $emitente);
            $this->validaSerie($ide["serie"]);
        }
    }
    
    /**
    *   Valida se o status da NFe é valido para a importação
    *
    *   @param int $statusNFe       - Status da NFe
    *   
    *   return int $statusNFe       - Retorna o status da NFe caso este sejá valido.
    */
    private function validaStatusNFe($nrAcessoNfe, $tpAmb, $tpEmis, $cdDigEstValue, $cnpj, $inscricao) {
        $statusNFe = 99;
        try {
            $statusNFe = $this->getStatusNFe($nrAcessoNfe, $tpAmb, $tpEmis, $cdDigEstValue, $cnpj, $inscricao);
        }
        catch(\Exception $e) {
            if($e->getMessage() == "Certificado Digital não encontrado." || $e->getMessage() == "Não há empresa vinculada ao CNPJ da nota.") {
                return 1;
            }
        }
        $statusNFe = 1;
        switch($statusNFe) {
            case 2:
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_CAN"]);
                break;
            case 3: 
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_DEN"]);
                break;
            case 5: 
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_DES"]);
                break;
            case 6: 
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_NRE"]);
                break;
            case 7: 
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["XML_DIF"]);
                break;
            case 99: 
                throw new \Exception(self::ERROS_IMPORTACAO["ERROS_MANIF"]["NAO_POS"]);
            default:
                return $statusNFe;
                break;
        }
    }
    
    /**
    *   Valida se o Fornecedor e o Destinatarios do XML são validos.
    *
    *   @param array $destinatario       - Dados do Destinatario
    *   @param array $emitente           - Dados do Emitente
    */
    private function validaFornecedorEDestinatario($destinatario, $emitente) {
        $cdFornecedor = $emitente["CPF"] ?? $emitente["CNPJ"] ?? '';
        
	    if($cdFornecedor) {
            $raiz = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaRaiz(substr($destinatario["CNPJ"], 0, 8));
            
            if($raiz == false) throw new \Exception(self::ERROS_IMPORTACAO["ERR_DESTINATARIO"]);
    	    else {
                $fornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getFornecedor($cdFornecedor);
                if(!isset($fornecedor[0])) {
                    $cadAutFornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCadAutFornecedor();
                    if($cadAutFornecedor['IDCADAUTFORNECED'] == 'S'){
                        //cadastra informações do fornecedor
                        if(isset($emitente["CPF"])){
                           $idTpijurforn =  'F';
                        }else if(isset($emitente["CNPJ"])){
                           $idTpijurforn =  'J'; 
                        }else if(isset($emitente["CODIGO"])){
                            $idTpijurforn =  'O'; 
                        }else{
                           $idTpijurforn =  'I';
                        }
                        $codFornecedor =  $idTpijurforn.$cdFornecedor;
                        $nrInsjurforn = $cdFornecedor;
                        $nome = $emitente["xNome"];
                        $nomeFantasia = $emitente["xFant"];
                        $nrIscest = $emitente["IE"];
                        $nrorg = $this->environment->getNrOrg();
                        
                        //informações do endereço do fornecedor
                        $cdPais = $emitente["enderEmit"]["cPais"];
                        $sgEstado = $emitente["enderEmit"]["UF"];
                        $cdMunicipio = $emitente["enderEmit"]["cMun"];
                        $dsEndForn = $emitente["enderEmit"]["xLgr"];
                        $nmBairro = $emitente["enderEmit"]["xBairro"];
                        $nrCep = $emitente["enderEmit"]["CEP"];
                        
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereFornecedor($codFornecedor,$idTpijurforn,$nrInsjurforn,$nome,$nomeFantasia,$nrIscest,$nrorg);
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereEndFornecedor($codFornecedor,$cdPais,$sgEstado,$cdMunicipio,$dsEndForn,$nmBairro,$nrCep,$nrorg);
                        
                    }else{
                        $mensagemErro = self::ERROS_IMPORTACAO["ERR_FORN"];
                        $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                        throw new \Exception($mensagemErro); 
                    }
                    
                }
    	        else {
                    $fornecedor = $fornecedor[0];
                    if($fornecedor["IDFORNIMPXML"] == "N") {
                        throw new \Exception(self::ERROS_IMPORTACAO["ERR_FORN_BLOQ"]);
                    }
    	        }
    	    }
	    }
    }
    
    /**
    *   Valida se o Fornecedor do CTE é valido.
    *
    *   @param array $cdFornecedor       - Dados do Destinatario
    */
    private function validaFornecedorCTe($cdFornecedor,$emitente) {
        $fornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getFornecedor($cdFornecedor);
        if(!isset($fornecedor[0])) {
            $cadAutFornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCadAutFornecedor();
            if($cadAutFornecedor['IDCADAUTFORNECED'] == 'S'){
                //cadastra informações do fornecedor
                if(isset($emitente["CPF"])){
                   $idTpijurforn =  'F';
                }else if(isset($emitente["CNPJ"])){
                   $idTpijurforn =  'J'; 
                }else if(isset($emitente["CODIGO"])){
                    $idTpijurforn =  'O'; 
                }else{
                   $idTpijurforn =  'I';
                }
                $codFornecedor =  $idTpijurforn.$cdFornecedor;
                $nrInsjurforn = $cdFornecedor;
                $nome = $emitente["xNome"];
                $nomeFantasia = $emitente["xFant"];
                $nrIscest = $emitente["IE"];
                $nrorg = $this->environment->getNrOrg();
                
                
                //informações do endereço do fornecedor
                $sgEstado = $emitente["enderEmit"]["UF"];
                $cdMunicipio = $emitente["enderEmit"]["cMun"];
                $cdPais = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCdPais($cdMunicipio);
                $cdPais = $cdPais["CDPAIS"];
                $dsEndForn = $emitente["enderEmit"]["xLgr"];
                $nmBairro = $emitente["enderEmit"]["xBairro"];
                $nrCep = $emitente["enderEmit"]["CEP"];
                
                
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereFornecedor($codFornecedor,$idTpijurforn,$nrInsjurforn,$nome,$nomeFantasia,$nrIscest,$nrorg);
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereEndFornecedor($codFornecedor,$cdPais,$sgEstado,$cdMunicipio,$dsEndForn,$nmBairro,$nrCep,$nrorg);
            }else{
                $mensagemErro = self::ERROS_IMPORTACAO["ERR_FORN"];
                $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                throw new \Exception($mensagemErro); 
            }
        }
        else {
            $fornecedor = $fornecedor[0];
            if($fornecedor["IDFORNIMPXML"] == "N") {
                throw new \Exception(self::ERROS_IMPORTACAO["ERR_FORN_BLOQ"]);
            }
            else {
                return array("IDTPIJURFORN" => $fornecedor["IDTPIJURFORN"], "CDFORNECED" => $fornecedor["CDFORNECED"]);
            }
        }
    }
    /**
    *   Valida se a série do arquivo xml existe no sistema
    * 
    *   @param string $cdSerie - Código da série da nota fiscal
    */
    private function validaSerie($cdSerie) {
        
        $serieExiste = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaSerieExiste($cdSerie);
        if($serieExiste == false) {
            $cadAutSerie = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCadAutSerie(); 
            if($cadAutSerie['IDCADAUTSERIE'] == 'S'){
               //cadastra informações da serie 
               $dsSerie = "Série ".$cdSerie;
               $nrorg = $this->environment->getNrOrg();
               
               $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereSerie($cdSerie,$dsSerie,$nrorg);
            }else{
                $mensagemErro = self::ERROS_IMPORTACAO["ERR_SERIE"];
                $mensagemErro = str_replace("%s", $cdSerie , $mensagemErro);
                throw new \Exception($mensagemErro);
            }
        }
    }
    
    /**
    *   Valida se o modelo de documento existe no sistema
    *
    *   @param string $modDocFisc       - Chave de Acesso da Nota Fiscal
    */
    protected function validaModeloDocumento($modDocFisc) {
        $modeloDocumento = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getModeloDocumento($modDocFisc);
        if(!isset($modeloDocumento[0])) {
            throw new \Exception(self::ERROS_IMPORTACAO["ERR_MODDOCFISC"]);
        }
    }
    
    
    /***************************************************************************
    *            Bloco de Funções referentes a manipulação de arquivos         *
    ****************************************************************************/
    
    /**
    *   Busca o nome dos arquivos no diretório parametrizado ou fileServer para a importação.
    * 
    *   @param string $extensao - Extensão do arquivo que sera procurado na pasta ('xml', 'xlsx' ou 'txt')
    *    
    *   return array $nomeArquivos - Array com nome dos arquivos
    */
    private function buscaNomeArquivosNotasImportacao($extensao = 'xml') {
        if($this->SAAS == false) {
            $diretorio = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDiretorio();   
            if(isset($diretorio[0])) {

                $diretorio = $diretorio[0]["DSDIRIMPARQXML"];
                $tam = strlen($diretorio);
                if(substr($diretorio, $tam-1, $tam) !== "/")
                    $diretorio = $diretorio."/";
                
                $this->diretorioImportacao = $diretorio;
                $this->criaDiretorios();
                
                $arquivos  = $this->getNomeArquivosDiretorio($diretorio, $extensao);
                if($arquivos)  return $arquivos; 
                else throw new \Exception(self::MSG_ERROS["NAO_NOT"]);
            }
            else {
                throw new \Exception(self::MSG_ERROS["SEM_DIR"]);
            }
        }
        else {
            $notas = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->existeNotaRelac();
            $arquivos = array();
        
            if(isset($notas[0])) {
                foreach($notas as $nota) {
                    array_push($arquivos, self::FILESERVER_LINK.$nota["FILESERVERLINK"]); 
                }
            }
            else throw new \Exception(self::MSG_ERROS["NAO_NOT"]);
        }
        
        return $arquivos;
    }
    
    /**
    *   Busca o nome de todos os arquivos de um diretório.
    *   
    *   @param string $diretorio - Diretório de onde os arquivos serão buscados 
    *    
    *   return array - Array com nome dos arquivos
    */
    private function getNomeArquivosDiretorio($diretorio, $extensao) {
        if(is_dir($diretorio)) {
            if($extensao == 'xml') {
                return glob("$diretorio{*.xml,*.XML}", GLOB_BRACE);
            }
            else {
                return glob("$diretorio{*.xlsx,*.XLSX}", GLOB_BRACE);        
                // return glob("$diretorio{*.xlsx,*.XLSX,*.txt,*.TXT}", GLOB_BRACE);                
            }
        }
        else {
            throw new \Exception(self::MSG_ERROS["DIR_INV"]);
        }
    }
    
    /**
    *   Cria os diretórios necessarios para a importação e que ainda não existem.
    */
    private function criaDiretorios() {
        $diretorios = array(
            "CTe" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe',
            "CTe/IMPORTADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'importados',
            "CTe/INVALIDOS"  => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'invalidos',
            "CTe/DUPLICADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'duplicados',
            "CTe/BLOQUEADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'bloqueados',
            "CTe/BLOQUEADOSFORN" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR.'bloqueados/fornecedores',
            "IMPORTADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'importados',
            "INVALIDOS"  => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'invalidos',
            "DUPLICADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'duplicados',
            "BLOQUEADOS" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'bloqueados',
            "BLOQUEADOSFORN" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'bloqueados/fornecedores'
        );
        
        try{
            foreach($diretorios as $diretorio) {
                if (!is_dir($diretorio)) {
                    if (!mkdir($diretorio, 0777, true)) {
                        throw new \Exception(
                            "Não foi possivel criar o diretorio $diretorio. Verifique as permissões"
                        );
                    }
                    chmod($diretorio, 0777);
                }
            }
        }catch(\Exception $e) {
            throw new \Exception("Permissão Negada"); 
        }
    
    }
    /**
    *   Cria o diretório para salvar um arquivo importado a partir da inscrição do destinatario
    *   
    *   @param array $destinatario - Dados do destinatario da nota
    */
    private function criaDiretorioSaveImportacao($destinatario) {
        
        $diretorios = array(
            "NFe" => $this->diretorioImportacao.DIRECTORY_SEPARATOR."importados",
            "CTe" => $this->diretorioImportacao.DIRECTORY_SEPARATOR.'CTe'.DIRECTORY_SEPARATOR."importados"
        );
        
        foreach($diretorios as $dir) {
            if(is_dir($dir)) {
                if($destinatario["IE"] != null) {
                    if(!is_dir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"])) {
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"], 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"], 0777);
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                    else if(!is_dir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'))) {
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                }
                else {
                    if(is_dir($dir.DIRECTORY_SEPARATOR.date('Y-m'))) {
                        mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                }
            }
            else {
                mkdir($dir, 0777, true);
                chmod($dir, 0777);
                if($destinatario["IE"] != null) {
                    if(!is_dir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"])) {
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"], 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"], 0777);
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                    else if(!is_dir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'))) {
                        mkdir($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.$destinatario["IE"].DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                }
                else {
                    if(is_dir($dir.DIRECTORY_SEPARATOR.date('Y-m'))) {
                        mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        chmod($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777);
                    }
                }
            }
        }
        
    }
    
    /***************************************************************************
    *            Bloco de Funções referentes a associação e validação          *
    *            de impostos e encargos                                        *
    ****************************************************************************/
    
    /**
    *   Verifica se os imposto da base estão associados e insere o código 
    *   destes impostos no array $impostos
    *   
    *   @param array $impostos
    */
    private function verificaAssociaImpostos(&$impostos) {
        $buscaImpostos = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getImpostos();
        
        if(isset($buscaImpostos[0])) {
            foreach ($buscaImpostos as $imposto) {
                switch ($imposto["SGIMPOSTONFE"]) {
                    case 'ICMS':
                        $impostos["ICMS"]       = $imposto["CDIMPOSTO"];
                        break;
                    case 'IPI':
                        $impostos["IPI"]        = $imposto["CDIMPOSTO"];
                        break;
                    case 'ISSQN-E':
                        $impostos["ISSQN"]    = $imposto["CDIMPOSTO"];
                        break;
                    case 'PIS':
                        $impostos["PIS"]        = $imposto["CDIMPOSTO"];
                        break;
                    case 'COFINS':
                        $impostos["COFINS"]     = $imposto["CDIMPOSTO"];
                        break;
                    case 'II':
                        $impostos["II"]         = $imposto["CDIMPOSTO"];
                        break;
                    case 'ICMS ST':
                        $impostos["STICMS"]     = $imposto["CDIMPOSTO"];
                        break;
                    case 'PIS ST':
                        $impostos["STPIS"]      = $imposto["CDIMPOSTO"];
                        break;
                    case 'COFINS ST':
                        $impostos["STCOFINS"]   = $imposto["CDIMPOSTO"];
                        break;
                }
            }
            
            foreach ($impostos as $nomeImposto => $cdImposto) {
                if(empty($cdImposto)) throw new \Exception(self::MSG_ERROS["IMP_ASO"]." Imposto $nomeImposto não associado.");
            }
        }
        else throw new \Exception(self::MSG_ERROS["IMP_ASO"]);
    }
    
    /**
    *   Verifica se os encargo da base estão associados e insere o código 
    *   destes encargos no array $encargos
    *    
    *   @param array encargos
    */
    private function verificaAssociaEncargos(&$encargos) {
        $buscaEncargos = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getEncargos();
         
        if(isset($buscaEncargos[0])) {
            foreach ($buscaEncargos as $encargo) {
                switch ($encargo["NMENCARGONFE"]) {
                    case 'Frete':
                        $encargos["Frete"]              = $encargo["CDENCARGO"];
                        break;
                    case 'Seguro':
                        $encargos["Seguro"]             = $encargo["CDENCARGO"];
                        break;
                    case 'Descontos':
                        $encargos["Descontos"]          = $encargo["CDENCARGO"];
                        break;
                    case 'Outras Despesas':
                        $encargos["Outras Despesas"]    = $encargo["CDENCARGO"];
                }
            }
            foreach ($encargos as $nomeEncargo => $cdEncargo) {
                if(empty($cdEncargo)) throw new \Exception(self::MSG_ERROS["ENC_ASO"]." Encargo $nomeEncargo não associado.");
            }
        }
        else throw new \Exception(self::MSG_ERROS["ENC_ASO"]);
    }
    
    /****************************************************************************
    *                        Bloco de Funções Auxiliares                        *
    ****************************************************************************/
    
    /**
    *   Insere o log em caso de erro na importação do arquivo XML
    *    
    *   @param string - Mensagem de erro
    *   @param string - Nome do arquivo importado
    */
    private function insereLogNota($mensagemErro, $nomeArquivo) {
        try {
            $this->erros[] = $mensagemErro;
            if($mensagemErro == self::ERROS_IMPORTACAO["ARQ_INV"] || $mensagemErro == "Versão do XML não suportada pelo sistema.") { 
                if($this->SAAS == false) {
                    $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                    $this->moveArquivoImportacao("invalidos", $nomeArquivo, '');
                }
                else {
                    $link = explode(self::FILESERVER_LINK, $nomeArquivo)[1];
                    $chave = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCdRelac($link)[0]["CDRELAC"];
                    $this->moveArquivoImportacao("invalidos", $nomeArquivo, $chave);
                    $nomeXml = $nrAcessoNfe.'.xml';
                }
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota('Arquivo Inválido',  'Interface not supported', $nomeXml);
            }
            else {
                $arquivo = file_get_contents($nomeArquivo);
                    
                $arrayXml = $this->converteXmlParaArray($arquivo);
                $nrAcessoNfe    = substr($arrayXml["NFe"]["infNFe"]["@attributes"]["Id"], 3);
                $chave          = $nrAcessoNfe;
        	    $numeroNf       = $arrayXml["NFe"]["infNFe"]["ide"]["nNF"];
        	    $serie          = $arrayXml["NFe"]["infNFe"]["ide"]["serie"];
                
                $emitente       = $arrayXml["NFe"]["infNFe"]["emit"];
                
                $cdFornecedor   = $emitente["CPF"] ?? $emitente["CNPJ"] ?? '';
                
        	    $nomeFornecedor = (isset($emitente["xNome"])) ? $emitente["xNome"] : '';
        	    
        	    $inscrEstadDest = isset($arrayXml["NFe"]["infNFe"]["dest"]["IE"]) ? $arrayXml["NFe"]["infNFe"]["dest"]["IE"] : $this->getInscrEstad($arrayXml["NFe"]["infNFe"]["dest"]["CNPJ"]);
                
                if($this->SAAS == false) {
                    $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                }
                else {
                    $nomeXml = $nrAcessoNfe.'.xml';
                }
                
                if($mensagemErro == self::ERROS_IMPORTACAO["ERROS_MANIF"]["XML_DIF"]) {
                    if(is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."invalidos") 
                        && !is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."invalidos/".DIRECTORY_SEPARATOR.$inscrEstadDest)) {
                            
                        mkdir($this->diretorioImportacao.DIRECTORY_SEPARATOR."invalidos/".DIRECTORY_SEPARATOR.$inscrEstadDest, 0777, true);
                    } 
                    $this->moveArquivoImportacao("invalidos".DIRECTORY_SEPARATOR.$inscrEstadDest, $nomeArquivo, $chave);
    
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == self::ERROS_IMPORTACAO["ERR_INS"]) {
                    $dir = $this->diretorioImportacao.DIRECTORY_SEPARATOR."duplicados/".DIRECTORY_SEPARATOR.$inscrEstadDest;
                    if(is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."duplicados")) {
                        if(!is_dir($dir)) {
                            mkdir($dir, 0777, true);
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                        else if(!is_dir($dir.DIRECTORY_SEPARATOR.date('Y-m'))) {
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                    } 
                    if($inscrEstadDest != null) {
                        $this->moveArquivoImportacao("duplicados".DIRECTORY_SEPARATOR.$inscrEstadDest.DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $chave);
                    }
                    else {
                        $this->moveArquivoImportacao("duplicados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $chave);
                    }
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == self::ERROS_IMPORTACAO["ERR_FORN_BLOQ"]) {
                    $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                    $this->moveArquivoImportacao("bloqueados/fornecedores", $nomeArquivo, $chave);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == str_replace("%s", $cdFornecedor, self::ERROS_IMPORTACAO["ERR_FORN"])) {
                    $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == str_replace("%s", $serie, self::ERROS_IMPORTACAO["ERR_SERIE"])) {
                    $mensagemErro = str_replace("%s", $serie , $mensagemErro);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if(    $mensagemErro == self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_CAN"]
                        ||  $mensagemErro == self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_DES"]
                        ||  $mensagemErro == self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_NAP"]
                        ||  $mensagemErro == self::ERROS_IMPORTACAO["ERROS_MANIF"]["ERR_NRE"]) {
                    
                    // Cria as pastas para inscrição estadual e para a data atual caso não existam
                    $dir = $this->diretorioImportacao.DIRECTORY_SEPARATOR."bloqueados/".DIRECTORY_SEPARATOR.$inscrEstadDest;
                    if(is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."bloqueados")) {
                        if(!is_dir($dir)) {
                            mkdir($dir, 0777, true);
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                        else if(!is_dir($dir.DIRECTORY_SEPARATOR.date('Y-m'))) {
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                    } 
    
                    if($inscrEstadDest != null) {
                        $this->moveArquivoImportacao("duplicados".DIRECTORY_SEPARATOR.$inscrEstadDest.DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $chave);
                    }
                    else {
                        $this->moveArquivoImportacao("duplicados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nomeArquivo);
                    }   
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, $numeroNf, $serie, $cdFornecedor, $nomeFornecedor);
                }
            }
        }
        catch(\Exception $e) {
            $this->erros[] = $e->getMessage()." ".$mensagemErro;   
        }
    }
    
    private function insereLogCTe($mensagemErro, $nomeArquivo) {
        try {
            $this->erros[] = $mensagemErro;
            
            if($mensagemErro == self::ERROS_IMPORTACAO["ARQ_INV"]) {   
                if($this->SAAS == false) {
                    $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                    $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."invalidos", $nomeArquivo, '');
                }
                else {
                    $link = explode(self::FILESERVER_LINK, $nomeArquivo)[1];
                    $chave = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getCdRelac($link)[0]["CDRELAC"];
                    $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."invalidos", $nomeArquivo, $chave);
                    $nomeXml = $nrAcessoNfe.'.xml';
                }
                $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota('Arquivo Inválido',  'Interface not supported', $nomeXml);
            }
            else {
                $arquivo  = file_get_contents($nomeArquivo);
                
                $xmlCTe  = new \DOMDocument("1.0", "UTF-8");
    		    $xmlCTe->loadXML($arquivo, LIBXML_NOBLANKS);
    		    
    		    $arrayXml  = $this->converteXmlParaArray($arquivo);
    		    $nrAcessoNfe = substr($arrayXml["CTe"]["infCte"]["@attributes"]["Id"], 3);
    		    
    		    if($this->SAAS == false) {
                    $nomeXml = explode($this->diretorioImportacao, $nomeArquivo)[1];
                }
                else {
                    $nomeXml = $nrAcessoNfe.'.xml';
                }
    		    
    		    $infCTe = $arrayXml["CTe"]["infCte"];
                $ide    = $infCTe["ide"];
                
                $serie    = $xmlCTe->getElementsByTagName('serie')->item(0)->nodeValue;
                $emit     = $infCTe["emit"];
                $cdFornecedor = isset($emit["CNPJ"]) ? $emit["CNPJ"] : $emit["CPF"];
                $nomeFornecedor   = $emit["xNome"];
                
                $toma = (isset($ide["toma3"])) ? $ide["toma3"]["toma"] : $ide["toma4"]["toma"];
                switch($toma) {
                    case 0:
                        $dest = $infCTe["rem"];
                        break;
                    case 1:
                        $dest = $infCTe["exped"];
                        break;
                    case 2:
                        $dest = $infCTe["receb"];
                        break;
                    case 3:
                        $dest = $infCTe["dest"];
                        break;
                    case 4:
                        $dest = $ide["toma4"];
                        break;
                }
                $cnpjDest = isset($dest["CNPJ"]) ? $dest["CNPJ"] : $dest["CPF"];
                $inscrEstadDest   = $dest["IE"] = isset($dest["IE"]) ? $dest["IE"] : $this->getInscrEstad($cnpjDest);
                
                if($mensagemErro == self::ERROS_IMPORTACAO["ERR_INS"]) {
                    $dir = $this->diretorioImportacao.DIRECTORY_SEPARATOR."CTe".DIRECTORY_SEPARATOR."duplicados/".DIRECTORY_SEPARATOR.$inscrEstadDest;
                    if(is_dir($this->diretorioImportacao.DIRECTORY_SEPARATOR."CTe".DIRECTORY_SEPARATOR."duplicados")) {
                        if(!is_dir($dir)) {
                            mkdir($dir, 0777, true);
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                        else if(!is_dir($dir.DIRECTORY_SEPARATOR.date('Y-m'))) {
                            mkdir($dir.DIRECTORY_SEPARATOR.date('Y-m'), 0777, true);
                        }
                    } 
                    
                    if($inscrEstadDest != null) {
                        $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."duplicados".DIRECTORY_SEPARATOR.$inscrEstadDest.DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $nrAcessoNfe);
                    }
                    else {
                        $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."duplicados".DIRECTORY_SEPARATOR.date('Y-m'), $nomeArquivo, $chave);
                    }
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, '', $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == self::ERROS_IMPORTACAO["ERR_FORN_BLOQ"]) {
                    $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                    $this->moveArquivoImportacao("CTe".DIRECTORY_SEPARATOR."bloqueados/fornecedores", $nomeArquivo, $chave);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, '', $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == str_replace("%s", $cdFornecedor, self::ERROS_IMPORTACAO["ERR_FORN"])) {
                    $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, '', $serie, $cdFornecedor, $nomeFornecedor);
                }
                else if($mensagemErro == str_replace("%s", $serie, self::ERROS_IMPORTACAO["ERR_SERIE"])) {
                    $mensagemErro = str_replace("%s", $serie , $mensagemErro);
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeXml, '', $serie, $cdFornecedor, $nomeFornecedor);
                }
            }
        }
        catch(\Exception $e) {
            $this->erros[] = $e->getMessage()." ".$mensagemErro;   
        }
    }
    
    /**
    *   Obtém o status da NFe.
    *   
    *   @param int    $nrAcessoNfe   - Número de acesso da  Nota Fiscal
    *   @param string $tpAmb         - Ambiente em que a nota foi transmitida
    *   @param string $tpEmis        - Tipo de Emissão
    *   @param int    $cdDigEstValue - Código do estado 
    *   @param string $cnpj          - CNPJ do destinatário
    *   @param string $inscricao     - Inscrição do destinatário
    *   
    *   return int - Status da NFe( 0 - Não encontrada; 1 - Aceita; 2 - Denegada
    *   3 - Cancelada; 4 - Confirmada Operação; 5 - Operação desconhecida; 6 - Operação não realizada
    *   7 - XML com dados distintos da SEFAZ; 8 - Ciência da Emissão; 9 - Impossivel Realizar a Consulta)
    */
    private function getStatusNFe($nrAcessoNfe, $tpAmb = 2, $tpEmis, $cdDigEstValue = null, $cnpj, $inscricao) {
        $manifestacao = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaManifestacao($nrAcessoNfe);

        // Notas existentes no sistema
        if(isset($manifestacao[0])) {
            $manifestacao = $manifestacao[0];
            if($manifestacao["IDSTATUSNFE"] == 2 || $manifestacao["IDSTATUSNFE"] == 3) {
                return $manifestacao["IDSTATUSNFE"];
            }
            else {  
                if($manifestacao["IDSTATUSMANIF"] == 2) return 5;      // Operação Desconhecida
                else if($manifestacao["IDSTATUSMANIF"] == 3) return 6; // Operação não Realizada
                else if($manifestacao["IDSTATUSMANIF"] == 4) return 8; // Ciencia da Emissão
                else if($manifestacao["IDSTATUSMANIF"] == 1) {
                    if($cdDigEstValue) { // Confirmada
                        if($manifestacao["CDDIGESTVALUE"] !== $cdDigEstValue) return 7; // XML com dados distintos da sefaz
                        else return 4; // Confirmação da Operação. Só é possivel em notas aceitas
                    }
                    else return 4; // Confirmação da Operação. Só é possivel em notas aceitas
                }
                else { // Sem Manifestação ou ciencia
                    if($cdDigEstValue) {
                        if($manifestacao["CDDIGESTVALUE"] !== $cdDigEstValue) return 7; // XML com dados distintos da sefaz
                        else return 1; // Aceita
                    }
                    else return 1; // Aceita
                }
            }
        }
        
        // Notas inexistentes no sistema
        else {
            $nrOrg   = $this->environment->getNrOrg();
            $empresa = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getEmpresaPorCnpj($cnpj);
            if(isset($empresa[0])) {
                $cdEmpresa = $empresa[0]["CDEMPRESA"];
            }
            else throw new \Exception("Não há empresa vinculada ao CNPJ da nota.");
            $config  = json_encode($this->manifestacaoService->montaConfigManifestacao($cdEmpresa, $nrOrg, $inscricao, $cnpj));
            
            $toolsNfe = $this->manifestacaoService->getToolsNFe($inscricao, $cnpj, $config);
            
            $config  = (array) json_decode($config);
            $nota = array();
            $this->aResposta = array();
            // Consulta a chave de acesso na sefaz e o retorno fica na variavel $nota
            $resp = $toolsNfe->sefazConsultaChave($nrAcessoNfe, $config["tpAmb"]);
            $this->aResposta = ReturnNFe::readReturnSefaz('zReadConsultaNF2', $resp);
        
            $tpEvento = '';
            $retorno = $this->aResposta;
            if($retorno["cStat"] == '101' || $retorno["cStat"] == '100') {
                if(count($retorno["aEvent"]) > 0) {
                    foreach($retorno["aEvent"] as $event) {
                        if($tpEvento != 3) {
                            if($event[0]["tpEvento"] == '110111') {
                                $tpEvento =  3; // Cancelada
                            }
                            else if($event[0]["tpEvento"] == '210200') {
                                $tpEvento =  4; // Confirmada a Operação
                            }
                            else if($event[0]["tpEvento"] == '210220') {
                                $tpEvento =  5; // Operação Desconhecida
                            }
                            else if($event[0]["tpEvento"] == '210240') {
                                $tpEvento =  6; // Operação Não Realizada
                            }
                            else if($event[0]["tpEvento"] == '210210') {
                                $tpEvento =  8; // Ciência da Emissão
                            }
                        }
                    }
                    if($tpEvento == '') {
                        if($retorno["aProt"]["cStat"] == '100' || $retorno["aProt"]["cStat"] == '150') {
                            $tpEvento =  1; // Aceita
                        }
                        else if($retorno["aProt"]["cStat"] == '110') {
                            $tpEvento =  2; // Denegada
                        }
                        else if($retorno["aProt"]["cStat"] == '101') {
                            $tpEvento =  3; // Cancelada
                        }
                        else {
                            $tpEvento =  0; // Não Encontrada
                        }
                    }
                }
                else {
                    if($retorno["aProt"]["cStat"] == '100' || $retorno["aProt"]["cStat"] == '150') {
                        $tpEvento =  1; // Aceita
                    }
                    else if($retorno["aProt"]["cStat"] == '110') {
                        $tpEvento =  2; // Denegada
                    }
                    else if($retorno["aProt"]["cStat"] == '101') {
                        $tpEvento =  3; // Cancelada
                    }
                    else {
                        $tpEvento =  0; // Não Encontrada
                    }
                }
                
                if(!empty($cdDigEstValue) && $cdDigEstValue !== $retorno["aProt"]["digVal"]) {
                    return 7;
                }
                else return $tpEvento;
            }
            
            else {
                $this->xMotivo = $retorno["xMotivo"];
                return 9;
            }
            
            /*$this->nfeTools = new ToolsNFe($config);
            $this->inicializaVarDiretorios();
    
            $notasBaixadas = $this->downloadAutomaticoXML($params['CDINSCESTA']);
    
            $aXml = $this->sincronizaNfe($params);
    
            return $this->aResposta;*/
        }
    }
    
    /**
    *   Converte uma string Xml para um array.
    *    
    *   @param string $stringXml
    *   return  array - Array do XML
    */
    protected function converteXmlParaArray($stringXml) {
        try{
            $xml = simplexml_load_string($stringXml,null,LIBXML_NOCDATA);
            $json = json_encode($xml);
            $array = json_decode($json,TRUE);
            return $array;
        }
        catch(\Exception $e){
            throw new \Exception(self::ERROS_IMPORTACAO["ARQ_INV"]);
        }
    }
    
    /**
    *   Move o Arquivo da pasta raiz de importação para outra pasta de acordo
    *   com o que ocorreu durante a importação.
    * 
    *   @param string $diretorioDestino         - Diretório para o qual o arquivo será movido
    *   @param string $nomeArquivo              - Nome do arquivo XML
    *   @param string $chave                    - Chave de acesso do arquivo
    *   @param string $novoNomeArquivo          - Nome do novo arquivo que será criado
    *   @param string $stringNota               - String da Nota Fiscal a ser importada
    */
    private function moveArquivoImportacao($diretorioDestino, $nomeArquivo, $chave, $novoNomeArquivo = '', $arquivo = '') {
        // Insereg o Arquivo na pasta de arquivos $diretorioDestino 
        if($this->SAAS == true) {
            $duplicado = explode("duplicados", $diretorioDestino);
            if(isset($duplicado[1])) {
                $inclusao = true;
                $cdRelac = $chave.'-duplicado';
                $existeDuplicado = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->buscaRelacFileServer($cdRelac);
                if(!isset($existeDuplicado[0])) {
                    $novoNomeArquivo = ($novoNomeArquivo != '') ? $novoNomeArquivo : $chave;
                    if($arquivo){
                        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->zGravaFile(1, $chave.'.xml', $arquivo, $diretorioDestino.'/', $inclusao, $cdRelac);
                    }else{
                        try{
                            $arquivo  = file_get_contents($nomeArquivo);
                        }catch(\Exception $e){
                            throw new \Exception(self::MSG_ERROS["SEM_DIR"]);
                        }
                    }
                }
            }
            else {
                $duplicado = explode("duplicados", $diretorioDestino);
                $cdRelac  = $chave;
                $inclusao = false;
                $novoNomeArquivo = ($novoNomeArquivo != '') ? $novoNomeArquivo : $chave;
                if($arquivo){
                    $this->dfe51000ImportacaoConsultaNotaFiscalFactory->zGravaFile(1, $chave.'.xml', $arquivo, $diretorioDestino.'/', $inclusao, $cdRelac);
                }else{
                    try{
                        $arquivo  = file_get_contents($nomeArquivo);
                    }catch(\Exception $e){
                        throw new \Exception(self::MSG_ERROS["SEM_DIR"]);
                    }
                }
            } 
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->deletaRelacFileServer($chave, 'importar');
        }
        else {
            $novoNomeArquivo = ($novoNomeArquivo != '') ? $novoNomeArquivo : explode($this->diretorioImportacao, $nomeArquivo)[1];
            if (!$arquivo) {
                $arquivo = file_get_contents($nomeArquivo);
            }
            $diretorioImportacao = $this->diretorioImportacao;
            if(substr($this->diretorioImportacao, strlen($this->diretorioImportacao) - 1) == DIRECTORY_SEPARATOR) {
                $diretorioImportacao = substr($this->diretorioImportacao, 0, strlen($this->diretorioImportacao) - 1);
            }
            else {
                $diretorioImportacao = $this->diretorioImportacao;
            }
            if(is_file($diretorioImportacao.DIRECTORY_SEPARATOR.$diretorioDestino.DIRECTORY_SEPARATOR.$novoNomeArquivo)) {
                unlink($diretorioImportacao.DIRECTORY_SEPARATOR.$diretorioDestino.DIRECTORY_SEPARATOR.$novoNomeArquivo);
            }
            file_put_contents($diretorioImportacao.DIRECTORY_SEPARATOR.$diretorioDestino.DIRECTORY_SEPARATOR.$novoNomeArquivo, $arquivo);
            // chmod($this->diretorioImportacao.DIRECTORY_SEPARATOR.$diretorioDestino.DIRECTORY_SEPARATOR.$novoNomeArquivo, 0777);
            // Remove o arquivo da pasta de arquivos importação
            if(is_file($nomeArquivo)) {
                unlink($nomeArquivo);
            }
        }
        
        
        // chmod($this->diretorioImportacao.DIRECTORY_SEPARATOR.$diretorioDestino.DIRECTORY_SEPARATOR.$novoNomeArquivo, 0777);
        // Remove o arquivo da pasta de arquivos em importação
        // unlink($nomeArquivo);
    }
    
    /**
    *  Salva o arquivo na no fileServer.
    */
    private function saveFileInFileServer() {
        
        $path = $file["path"];
        $path = ($mode === 'prod') ? self::PROD_LINK.$path : self::DEV_LINK.$path;
        $data = file_get_contents($path);
        
        openssl_pkcs12_read($data, $cert, $senha);
        $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'certKey'.uniqid().'.pem';
        $certKey = file_get_contents($tmpFile);
        $pem = base64_encode($certKey);

        $pem = 'data: '.mime_content_type($tmpFile).';base64,'.$pem;
        
        $lastModify = filectime($tmpFile);

        $filePem = array(
            "name" => "certKey.pem", 
            "lastModified" => $lastModify,
            "lastModifiedDate" => '',
            "webkitRelativePath" => '',
            "type" => '',
            "size"  => filesize($tmpFile),
            "b64File" => $pem);
            
        $row['CERTKEY']    = array();
        $row['CERTKEY'][0] = $filePem;
        $this->fileUploader->uploadFilesInRow($row, 'CERTKEY', 'Certificados');
        unlink($tmpFile);
        return $row['CERTKEY'][0]['path'];
    }
    
    /**
    *   Retorna a sigla do estado de acordo com o código recebido
    *    
    *   @param int $codigo - Codigo do estado
    *   return string - Nome do Estado
    */
    public function getSiglaEstado($codigo) {
        switch ($codigo) {
            // Região Norte
            case 11:    return 'RO'; break; 
            case 12:    return 'AC'; break;
            case 13:    return 'AM'; break;
            case 14:    return 'RR'; break;
            case 15:    return 'PA'; break;
            case 16:    return 'AP'; break;
            case 17:    return 'TO'; break;
            
            // Região Nordeste
            case 21:    return 'MA'; break;
            case 22:    return 'PI'; break;
            case 23:    return 'CE'; break;
            case 24:    return 'RN'; break;
            case 25:    return 'PB'; break;
            case 26:    return 'PE'; break;
            case 27:    return 'AL'; break;
            case 28:    return 'SE'; break;
            case 29:    return 'BA'; break;
            
            //Região Sudeste
            case 31:    return 'MG'; break;
            case 32:    return 'ES'; break;
            case 33:    return 'RJ'; break;
            case 35:    return 'SP'; break;
            
            //Região Sul
            case 41:    return 'PR'; break;
            case 42:    return 'SC'; break;
            case 43:    return 'RS'; break;
            
            //Região Centro-Oeste
            case 50:    return 'MS'; break;
            case 51:    return 'MT'; break;
            case 52:    return 'GO'; break;
            case 53:    return 'DF'; break;
        }
    }
    
    /** 
    *   Formata uma data no padrão brasileiro d/m/Y
    *   @param \DateTime $data - Data a ser formatada
    *  
    *   return \DateTime - Data formatada em d/m/Y
    */
    public function formataData($data) {
        return date("d/m/Y", strtotime($data));
    }
    
    /** 
    *   Remove os acentos de determinada String
    *   @param string $string
    *  
    *   return string - String sem os acentos
    */   
    public function removerAcentos($string){
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/"),explode(" ","a A e E i I o O u U n N"),$string);
    }
    
    public function getInscrEstad($nrinsjuriest) {
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getInscrEstad($nrinsjuriest);
    }
    
    public function getErros() {
        return $this->erros;
    }

    public function getDefaultFilterImportacao($row) {
        $cdfilial = ($this->environment->getCdFilial()) ? $this->environment->getCdFilial() : $row["CDFILIAL"];
        $defaultFilter = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDefaultFilterImportacao($cdfilial);
        $defaultFilter["DTEMISSAO"] = date("d/m/Y");
        return $defaultFilter;
    }
    
    private function getHtmlItensPendentes($itens) {
        $oldCdFornecedor = '';
        $oldEmailItemPen = '';
        
        $htmlPorDestinatario = array();
        
        foreach ($itens as $item) {
            $cdProdXml       = $item["CDPRODUTO"];
            $nmProduto       = $item["NMPRODUTO"];
            $fornecedor      = (strlen($item["NRINSJURFORN"]) == 14) ? $this->mask($item["NRINSJURFORN"], "##.###.###/####-##") : $this->mask($item["NRINSJURFORN"], "###.###.###-##");
            $nmFornecedor    = $item["NMRAZSOCFORN"]; 
            
            if($oldCdFornecedor != $item["CDFORNECED"]) {
                if($item["DSEMAILITPEN"] != $oldEmailItemPen) {
                    if($oldEmailItemPen != '') {
                        $htmlEmail.=$this->getFooterEmail();
                        $htmlPorDestinatario[$oldEmailItemPen]["HTML"] = $htmlEmail;
                        $htmlPorDestinatario[$oldEmailItemPen]["CDFORNECED"] = $oldCdFornecedor;
                        $htmlEmail = $this->getHeaderEmail();
                        
                    }
                    else {
                        $htmlEmail = $this->getHeaderEmail();
                    }
                }
                if($oldCdFornecedor != '' && ($item["DSEMAILITPEN"] == $oldEmailItemPen)) {
                    $htmlEmail.= "</div>";
                    $htmlEmail.=  "<div>";
                }
                else {
                    $htmlEmail.=  "<div>";
                }
                $oldCdFornecedor = $item["CDFORNECED"];
                $oldEmailItemPen = $item["DSEMAILITPEN"];
                $htmlEmail.= "<p class='forn'><b>Fornecedor</b>: $fornecedor - $nmFornecedor</p>";
                $class = 'firstItem';
            }
            else{
                $class = 'item';
            }
            $htmlEmail.= "<div class='$class'>
                             <b>&nbsp;&nbsp;Código do produto</b>: $cdProdXml<br>
                             <b>&nbsp;&nbsp;Nome do produto</b>: $nmProduto
                         </div>";
        }
           
        $htmlEmail.=$this->getFooterEmail();
        
        $htmlPorDestinatario[$oldEmailItemPen]["HTML"] = $htmlEmail;
        $htmlPorDestinatario[$oldEmailItemPen]["CDFORNECED"] = $oldCdFornecedor;
        
        return $htmlPorDestinatario;
    }


    private function getHeaderEmail() {
        return '<html>
            <body class="body">
                <style>
                    .forn {
                        border-top: 1px solid #000;
                        padding-left: 2px;
                        font-size: 16px;
                    }
                    .firstItem {
                        padding-left: 2px;
                        font-size: 14px;
                    }
                    .item {
                        padding-top: 5px;
                        padding-bottom: 5px;
                        padding-left: 2px;
                        font-size: 14px;
                    }
                    .body {
                        display: block;
                    }
                    .container{
                        border: 1px solid #cccaca;
                        background-color: #dbdbdb;
                        border-bottom: 1px solid #000;
                        border: 1px solid #cccaca;
                        font-family: "Segoe UI","Lucida Sans",sans-serif;
                    }
                    .text-center {
                        width: 92%;
                        left: 4%;
                        right: 4%;
                        position: relative;
                    }
                    .description {
                        font-size: 19px;
                        text-align: center;
                    }
                    .footer {
                        padding-bottom: 2px;
                        border-top: 1px solid #000;
                        color: #00528C;
                        max-height: 50px;
                        height: 20%;
                        text-align: center;
                    }
                </style>
                <div class="container">
                    <div class="text-center">
                        <p class="description"><b>Itens Pendentes de Relacionamento Produto x Produto XML</b></p>';
    }
    
    private function getFooterEmail() {
        return ' 
                        <br>
                            <div class="footer">
                                <p class="textFooter">Email automático enviado pela Importação de XML</p>
                            </div>
                        </div>
                    </div>
                </div>
            </body>
        </html>';
    }

    private function enviarEmailItensPendente($htmlEmail, $destinatarios, $dadosSmtp) {
        $this->smtpService = new SMTP(['host'          => $dadosSmtp['DSSMTPEMAILF'],
                                       'username'      => $dadosSmtp['DSLOGINSMTP'],
                                       'password'      => $this->getDelphiCryptoUncripto($dadosSmtp['CDSENHEMAILF'], false),
                                       'port'          => isset(explode(":", $dadosSmtp['DSSMTPEMAILF'])[1]) ? explode(":", $dadosSmtp['DSSMTPEMAILF'])[1] : '',
                                       'smtpSecure'    => $dadosSmtp['IDUTTLSEMAIL'] == 'S' ? 'tls' :  ($dadosSmtp['IDUTSSLEMAIL'] == 'S' ?  'ssl' : ''),
                                       'from'          => $dadosSmtp['DSLOGINSMTP'] ]);
        $fromname = 'Importação XML';
        $destinatarios = explode(";", $destinatarios);
        $assunto   = 'Itens Pendentes de Relacionamento Produto x Produto XML';
        $this->smtpService->setFromName($fromname);
        $this->smtpService->setSubject($assunto);
        $enviado  = $this->smtpService->sendMail($htmlEmail, $destinatarios, ''); // 4º Parametro - Copia Oculta
        
        return $enviado;
    }
    
    public static function getDelphiCryptoUncripto($password, $isEncrypt = true) {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = ['encoding'    => 'ISO-8859-1',
                    'trace'       => true,
                    'exceptions'  => true,
                    'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP];
        
        $args = [new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
                 new \SoapVar('2', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")];
        //2
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUnCripto';
        $params = ['rdm'    => 'Bib.rdmDBGeral',
                   'method' => $cryptType,
                   'args'   => $args];
        
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            return 'SoapError';
        }
        return $encryption;
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    public function mask($val, $mask) {
         $maskared = '';
         $k = 0;
        for($i = 0; $i<=strlen($mask)-1; $i++) {
            if($mask[$i] == '#') {
                if(isset($val[$k])) {
                    $maskared .= $val[$k++];
                }
            }
            else{
                if(isset($mask[$i])){
                    $maskared .= $mask[$i];
                }
            }
        }
        return $maskared;
    }
    
    public function enviaEmailRelacionamentoPendente($entityManager = null) {
        try {
            $dadosSmtp = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getDadosSmtpItensPendentes($entityManager);
            if($dadosSmtp) {
                
                $itens = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getItensPendentesParametrizacao($entityManager);
                if($itens) {
                    $htmlPorDestinatario  = $this->getHtmlItensPendentes($itens);
              
                    foreach ($htmlPorDestinatario as $destinatario => $item) {
                        $emailEnviado = false;
                        $emailEnviado = $this->enviarEmailItensPendente($item["HTML"], $destinatario, $dadosSmtp); 
                        
                        if($emailEnviado) $this->dfe51000ImportacaoConsultaNotaFiscalFactory->updateEmailItemPendente($item["CDFORNECED"], $entityManager);
                    }
                    return ['SUCCESS' => 'Envio de e-mail dos itens pendente de relacionamento produto x produto xml realizado com sucesso!'];
                }
                return ['Nenhum e-mail a enviar de itens pendente de relacionamento de produto.'];
                
            } else throw new \Exception('Configuração de SMTP não realizada para a maior unidade cadastrada.');
            
        } catch (\Exception $e) {
            return ['ERRORS' => $e->getMessage()];
        }
    }
    
    /*public function callEnviarEmailNfPendente() {
        $requestUri = $_SERVER["REQUEST_URI"];
        
        if(strpos($requestUri, "modules") == false) {
            $urlEmailPendente = $_SERVER["HTTP_HOST"].'/backend_df/index.php/enviaEmailRelacionamentoPendente';
        }
        else {
            if(strpos($_SERVER["DOCUMENT_ROOT"], "workfolder") != false) {
                $urlEmailPendente = $_SERVER["HTTP_HOST"].'/workfolder'.$_SERVER["REQUEST_URI"];
            }
            else {
                $urlEmailPendente = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
            }
        }
        $urlEmailPendente = 'http://'.str_replace("importarNotasFiscais", "enviaEmailRelacionamentoPendente", $urlEmailPendente);
        
        $file = __DIR__.DIRECTORY_SEPARATOR.'EnviaEmailPendente.php';
        
        exec( "php $file $urlEmailPendente > /tmp/logEnviaEmailPendete.txt &");
    }*/
    
    public function atualizaStatusXml($nrAcessoNfe, $idStatusNfe) {
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->atualizaStatusXml($nrAcessoNfe, $idStatusNfe);
    }                                                                                                                                               

    public function insereXml($params) {
        return $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereXml($params);
    }    
    
    public function getSequencialHistoricoXML($nrAcessoNfe){
        
        $sequencial = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getSequencialHistoricoXML($nrAcessoNfe)[0]["NRSEQHISTOR"];

        if($sequencial === null) $sequencial = 1;
        else{
            (int)$sequencial += 1;
        }

        strval($sequencial);
        $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
         
        return $sequencial;   
    }
    

    
    /***************************************************************************
    *    Bloco de Funções referentes a importação da Facturacion Electrônica    *
    ****************************************************************************/
    
    /**
    *   Realiza a leitura de uma facturacion do tipo planilha xlsx 
    *   e a converte em um array.
    *   
    *   @param string $nomeArquvio      - Nome do arquivo a ser importado
    */
    protected function lerFacturacionXLSX($nomeArquivo) {
        $excelReader = \PHPExcel_IOFactory::createReaderForFile($nomeArquivo);
        $arquivo = $excelReader->load($nomeArquivo);
        
        // Lê planilha sem index para os valores 
        $arquivo = $arquivo->getActiveSheet()->toArray(null, true,true,true);

        $nomeColunas = $arquivo[1];
        $count = 0;
        // Cria o indice para os valores
        $notas = array();
        foreach($arquivo as $linha) {
            if($count != 0) {
                foreach($linha as $coluna => $dadoColuna) {
                    $notas[$count - 1][$nomeColunas[$coluna]] = $dadoColuna; 
                }
            }
            $count ++;
        }
        
        return $notas;
    }
    
    /**
    *   Realiza a leitura de uma facturacion do tipo txt 
    *   e a converte em um array.
    *   
    *   @param string $nomeArquvio      - Nome do arquivo a ser importado
    */
    protected function lerFacturacionTXT($nomeArquivo) {
        $arquivo = file_get_contents($nomeArquivo);
        $arquivo = explode("\n", $arquivo);
        
        $nomeColunas = $arquivo[0];
        
        $nomeColunas = explode("|", $nomeColunas);


        $count = 0;
        // Cria o indice para os valores
        $notas = array();
        foreach($arquivo as $linha) {
            $linha = explode("|", $linha);
            if($count != 0) {
                foreach($linha as $coluna => $dadoColuna) {
                    $notas[$count - 1][$nomeColunas[$coluna]] = $dadoColuna; 
                }
            }
            $count ++;
        }
        
        return $notas;
    }
    
    /**
    *   Importa o nota fiscal colombiana a partir de um array.
    *   
    *   @param string $nomeArquvio          - Nome do arquivo a ser importado
    *   @param array  $nota                 - Dados da nota a ser importada
    *   @param string $operador             - Código do operador que fez a operação
    *   @param float  $totalProdutos        - Valor total dos produtos da nota
    *   @param int    &$contImp             - Contador de notas com sucesso na importação
    *   @param int    &$contErro            - Contador de erros
    *   @param array  &$notasImportadas     - Notas que já foram importadas 
    *   @param array  $impostosFacturacion  - Dados dos impostos da facturacion 
    */
    public function importarFacturacionColombia($nomeArquivo, $nota, $operador, $totalProdutos, &$contImp, &$contErro, $notasImportadas, $impostosFacturacion) {
        
        if(!isset($notasImportadas[$nota["CFD_campo_texto_20"]])) {
            $this->insereCabecalhoFacturacion($nota, $nomeArquivo, $totalProdutos);
            $notasImportadas[$nota["CFD_campo_texto_20"]] = 1;
        }
        
        $contadorProduto = $notasImportadas[$nota["CFD_campo_texto_20"]];
        
        $this->insereItemFacturacion($contadorProduto, $nota["CodigoServientre"], $nota["NombreServientre"], $nota["CON_unidad_medida_con"], $nota["CajasTotal"],
                                (float) $nota["DeclaredValue_COP"], (float) $nota["DescuentoPorItem"], (float) $nota["TotalConcepto"], $nota["CFD_campo_texto_20"]);
        

        if($impostosFacturacion["IVA"]["IDIMPORTAIMPXML"] == 'S') {
            $this->insereImpostoItemFacturacion($contadorProduto, $impostosFacturacion["IVA"]["CDIMPOSTO"], $nota["TotalFactura"], $nota["ConceptoPorIVA"],
                                            $nota["TotalIVA"], $nota["CFD_campo_texto_20"]);
        }
        if($impostosFacturacion["IMPOCONSUM"]["IDIMPORTAIMPXML"] == 'S') {
            $this->insereImpostoItemFacturacion($contadorProduto, $impostosFacturacion["IMPOCONSUM"]["CDIMPOSTO"], $nota["TotalFactura"], $nota["ConceptoPor1"],
                                            $nota["Con_Impuesto2"], $nota["CFD_campo_texto_20"]);
        }
                                
        $notasImportadas[$nota["CFD_campo_texto_20"]] += 1;
        
    }
    
    /**
     *  Valida se os impostos relativos a facturacion electrônica possuem
     *  relacionamento no sistema
     * 
     *  @param array $impostosFacturacion   - Impostos da facturacion electrônica
     * 
     *  return array $impostos - Impostos 
     */
    protected function validaImpostosFacturacion($impostosFacturacion) {

        $impostos = array();
        foreach ($impostosFacturacion as $imposto) {
            if($imposto["SGIMPOSTONFE"] == 'IVA') {
                $impostos["IVA"] = $imposto;
            }
            else if($imposto["SGIMPOSTONFE"] == 'IMPOCONSUM') {
                $impostos["IMPOCONSUM"] = $imposto;
            }
        }
        if(!isset($impostos["IVA"]) || !isset($impostos["IMPOCONSUM"])) throw new \Exception(self::MSG_ERROS["IMP_FAT"]);
        
        return $impostos;
    }
    
    /**
     *  Insere o cabeçalho da nota fiscal no banco de dados 
     * 
     *  @param array $nota          - Dados da nota fiscal
     *  @param string $nomeArquivo  - Nome do arquivo da nota fiscal 
     */
    protected function insereCabecalhoFacturacion($nota, $nomeArquivo, $totalProdutos) {
        $cdFornecedor = $nota["ID"];
        $cdFornecedor = str_pad($cdFornecedor, 20, '0', STR_PAD_LEFT);
        $fornecedor = $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getFornecedor($cdFornecedor);
        
        $this->validacoesCabecalhoFacturacion($nota);
        
        if(!isset($fornecedor[0])) {
            throw new \Exception(self::ERROS_IMPORTACAO["ERR_FORN"]);
        }
        else {
            $fornecedor = $fornecedor[0];
        }

        $dtEmissao  = \DateTime::createFromFormat('m/d/y', str_replace("-", "/", substr($nota["PaymentDate"], 0, 10)))->format('d/m/Y');
        $dtEntrSaid = (isset($nota["pague_hasta3"])) ?  \DateTime::createFromFormat('m/d/y', str_replace("-", "/", substr($nota["pague_hasta3"], 0, 10)))->format('d/m/Y') : '';

        $nrInsJurEmpr  = (isset($emitente["CNPJ"])) ? $emitente["CNPJ"] : '';
        
        $nomeXmlForne = explode("/", $nomeArquivo);
        $nomeXmlForne = $nomeXmlForne[count($nomeXmlForne) - 1];
        
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgNotaFiscal('', '', '', '', $fornecedor["IDTPIJURFORN"],
            $cdFornecedor, $fornecedor["CDFORNECED"], $nota["OrderNumber"], $dtEmissao, $dtEntrSaid, $totalProdutos, (float) $nota["TotalFactura"], 'U', '', '', '', $nota["CFD_campo_texto_20"],
            'N', '', '1', '1', '', '', $nota["DocType"], $nomeXmlForne, '', 
            '', $nota['CFD_campo_texto_21'], '',  '', '', '', '', '', '', '', $this->environment->getNrOrgTrab());
        
    }
    
    /*
    *  Realiza as validações necessarias para a importação da facturacion
    *
    *  @param array $nota          - Dados da nota fiscal
    **/
    protected function validacoesCabecalhoFacturacion($nota) {
        $this->validaSerie('U');
        $this->validaModeloDocumento($nota["DocType"]);
        if($this->dfe51000ImportacaoConsultaNotaFiscalFactory->verificaNotaImportada($nota["CFD_campo_texto_20"]))
            throw new \Exception(self::ERROS_IMPORTACAO["ERR_INS"]);
    }
    
    /**
     *  Insere o produto da facturacion electronica
     * 
     *  @param int $contadorProduto     - Contador sequencial dos produtos
     *  @param string $codigoProduto    - Código do produto
     *  @param string $nomeProduto      - Nome do produto
     *  @param string $unidadeMedida    - Unidade de medida do produto
     *  @param float $quantidade        - Quantidade do produto na nota
     *  @param float $valorUnitario     - Valor unitario do produto na nota
     *  @param float $desconto          - Desconto do produto na nota
     *  @param float $valorTotal        - Valor total do produto na nota
     *  @param string $chaveAcesso      - Chave de acesso a qual o produto está relacionado
     */    
    protected function insereItemFacturacion($contadorProduto, $codigoProduto, $nomeProduto, $unidadeMedida, 
                                                $quantidade, $valorUnitario, $desconto, $valorTotal, $chaveAcesso) {
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgItemProduto('', '', $contadorProduto, $codigoProduto, $nomeProduto, $unidadeMedida,
                        $quantidade, $valorUnitario, $desconto, $valorTotal, '', '', '', '', '', '', '', '', '', 
                        $chaveAcesso, '', '', '', $this->environment->getNrOrgTrab());
    }
    
    /**
     *  Insere o imposto da facturacion electronica
     * 
     *  @param int $contadorProduto     - Contador sequencial dos produtos ao qual o imposto está relacionado
     *  @param string $codigoImposto    - Código do imposto
     *  @param float $valorBc           - Valor da base de calculo do imposto
     *  @param float $percentualImposto - Percentual do imposto sobre o valor da nota
     *  @param float $totalImposto      - Valor total do imposto da nota
     *  @param string $chaveAcesso      - Chave de acesso a qual o produto está relacionado
     */  
    protected function insereImpostoItemFacturacion($contadorProduto, $codigoImposto, $valorBc, $percentualImposto, $totalImposto, $chaveAcesso) {
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereIntgTrabItProImpos('', '', $contadorProduto, $codigoImposto,
                                    '', '', $valorBc, $percentualImposto, $totalImposto,  '', 
                                    '', '', '1', $chaveAcesso, '', '', '', '', $this->environment->getNrOrgTrab());
    }
    
    /**
     *  Insere o log da facturacion em caso de erros
     * 
     *  @param string $mensagemErro     - Mensagem de erro
     *  @param string $nomeArquivo      - Nome do arquivo importado
     *  @param array  $nota             - Dados da facturacion
     */
    protected function insereLogFacturacion($mensagemErro, $nomeArquivo, $nota) {
        
        $nrAcessoNfe    = $nota["CFD_campo_texto_20"];
        $cdFornecedor   = $nota["CodigoServientre"];
        $nomeFornecedor = $nota["NombreServientre"];
        
        $this->erros[] = $mensagemErro; 
        
        if($mensagemErro == self::ERROS_IMPORTACAO["ERR_FORN"]) {
            $mensagemErro = str_replace("%s", $cdFornecedor , $mensagemErro);
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeArquivo, '', '', $cdFornecedor, $nomeFornecedor);
        }
        else if($mensagemErro == self::ERROS_IMPORTACAO["ERR_SERIE"]) {
            $mensagemErro = str_replace("%s", 'U' , $mensagemErro);
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeArquivo, '', '', $cdFornecedor, $nomeFornecedor);
        }
        else if($mensagemErro == self::ERROS_IMPORTACAO["ERR_MODDOCFISC"]) {
            $mensagemErro = str_replace("%s", $nota["DocType"] , $mensagemErro);
            $this->dfe51000ImportacaoConsultaNotaFiscalFactory->insereLogNota($nrAcessoNfe, $mensagemErro, $nomeArquivo, '', '', $cdFornecedor, $nomeFornecedor);
        }
    }
    public function limpaLoginXML(){
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->limpaLoginXML();
    }
    
    public function getManifestacaoService() {
        return $this->manifestacaoService;
    }
    
    public function setAgendamentoImportacao($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd){
            $this->entityManager = $newEntityManager;
            $this->connection = $newEntityManager->getConnection();
        }
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
        $this->dfe51000ImportacaoConsultaNotaFiscalFactory->setAgendamentoImportacao($nrorg, $useVpd, $newEntityManager, $newFileUploader);
    }
}